@@ -20,8 +20,13 @@ melvq; % mbest VQ functions
 % General 2nd order parabolic interpolator.  Used splines orginally,
 % but this is much simpler and we don't need much accuracy.  Given two
 % vectors of points xp and yp, find interpolated values y at points x
+%
 
-function y = interp_para(xp, yp, x)
+% If a point in x is less than the smallest point in xp, we linearly
+% interpolate down to (0,0).  If a point in x is greater than the
+% greatest value in xp, we linearly interpolate down to (xp_max, 0)
+
+function y = interp_para(xp, yp, xp_max, x)
   assert( (length(xp) >=3) && (length(yp) >= 3) );
 
   y = zeros(1,length(x));
@@ -31,21 +36,86 @@ function y = interp_para(xp, yp, x)
 
     % k is index into xp of where we start 3 points used to form parabola
 
-    while ((xp(k+1) < xi) && (k < (length(xp)-2))) 
+    while (xp(k) < xi) && (k < length(xp)) 
       k++;
     end
     
-    x1 = xp(k); y1 = yp(k); x2 = xp(k+1); y2 = yp(k+1); x3 = xp(k+2); y3 = yp(k+2);
-    %printf("k: %d i: %d xi: %f x1: %f y1: %f\n", k, i, xi, x1, y1);
+    %printf("xi: %f k = %d\n", xi, k);
+    if k == 1
+      % linear interpolation at low end
+      x1 = 0; y1 = 0;
+      x2 = xp(k); y2 = yp(k); 
+      b = (y2-y1)/(x2-x1);
+      y(i) = b*(xi-x2) + y2;
+      %printf("lin1 k: %d i: %d xi: %f x1: %f y1: %f\n", k, i, xi, x1, y1);
+    elseif k < length(xp)
+      % parabolic interpolation
+      x1 = xp(k-1); y1 = yp(k-1); 
+      x2 = xp(k); y2 = yp(k); 
+      x3 = xp(k+1); y3 = yp(k+1);
+      a = ((y3-y2)/(x3-x2)-(y2-y1)/(x2-x1))/(x3-x1);
+      b = ((y3-y2)/(x3-x2)*(x2-x1)+(y2-y1)/(x2-x1)*(x3-x2))/(x3-x1);
+      y(i) = a*(xi-x2)^2 + b*(xi-x2) + y2;
+      %printf("para1 k: %d i: %d xi: %f x1: %f y1: %f\n", k, i, xi, x1, y1);
+    elseif (k == length(xp)) && (xi < xp(k))
+      % parabolic interpolation, but shift xp points back by 1
+      x1 = xp(k-2); y1 = yp(k-2); 
+      x2 = xp(k-1); y2 = yp(k-1); 
+      x3 = xp(k); y3 = yp(k);
+      a = ((y3-y2)/(x3-x2)-(y2-y1)/(x2-x1))/(x3-x1);
+      b = ((y3-y2)/(x3-x2)*(x2-x1)+(y2-y1)/(x2-x1)*(x3-x2))/(x3-x1);
+      y(i) = a*(xi-x2)^2 + b*(xi-x2) + y2;
+      %printf("para2 k: %d i: %d xi: %f x1: %f y1: %f\n", k, i, xi, x1, y1);
+    elseif k == length(xp)
+      % linear interpolation at high end
+      x1 = xp(k); y1 = yp(k); 
+      x2 = xp_max; y2 = 0;
+      b = (y2-y1)/(x2-x1);
+      y(i) = b*(xi-x1) + y1;
+      %printf("lin2 k: %d i: %d xi: %f x1: %f y1: %f\n", k, i, xi, x1, y1);
+    end
 
-    a = ((y3-y2)/(x3-x2)-(y2-y1)/(x2-x1))/(x3-x1);
-    b = ((y3-y2)/(x3-x2)*(x2-x1)+(y2-y1)/(x2-x1)*(x3-x2))/(x3-x1);
-  
-    y(i) = a*(xi-x2)^2 + b*(xi-x2) + y2;
   end
 endfunction
 
 
+% choose largest sample in band, idea is we care more about finding
+% peaks, can handle some error in frequency. x are non linear
+% (arbitrary) sampling points in kHz
+
+function y = interp_largest(f0_Hz, AmdB, x_kHz)
+  L = length(AmdB);
+  x = x_kHz*1000;
+  y = zeros(1,length(x));
+  bw = x(2) - x(1);
+  k = 1;
+
+  for i=1:length(x)
+
+    % determine limits of this band
+
+    if i>1
+      bw = x(i) - x(i-1);
+    end
+    band_low = x(i) - bw/2; band_high = x(i) + bw/2;
+
+    % map band limits to harmonics    
+ 
+    if x(i) < f0_Hz
+      m_low = m_high = 1;
+    else
+      m_low = round(band_low/f0_Hz); m_high = round(band_high/f0_Hz)-1;
+      m_low = max(1, m_low); m_high = min(L, m_high); m_high = max(m_low, m_high);
+    end
+
+    printf("L: %d f0: %f i: %d band_low: %f band_high: %f m_low: %d m_high: %d\n",L, f0_Hz, i, band_low, band_high, m_low, m_high);
+    % find max in band
+
+    y(i) = max(AmdB(m_low:m_high));
+  end
+
+endfunction
+
 % simple linear interpolator
 
 function y = interp_linear(xp, yp, x)
@@ -130,13 +200,241 @@ function value = bits_to_index(bits, numbits)
 endfunction
 
 
+% generate a zig-zag linear to square mapping matrix
+
+function map = create_zigzag_map(nr,nc)
+  map = zeros(nr, nc);
+
+  state = 'zig';
+  r = c = 1;
+
+  for i=1:nr*nc
+
+    printf("%s r: %d c: %d i %d\n", state, r, c, i);
+    map(r,c) = i;
+
+    next_state = state;
+    if state == 'zig'
+      % move SE
+      c -= 1; r += 1;
+      if r > nr
+        r = nr; c+=2;
+        next_state = 'zag';
+      end
+      if c < 1
+        c = 1;
+        next_state = 'zag';
+      end
+    end
+
+    if state == 'zag'
+      % move SE
+      r -= 1; c +=1;
+      if c > nc
+        c = nc; r+=2;
+        next_state = 'zig';
+      end
+      if r < 1
+        r = 1;
+        next_state = 'zig';
+      end
+    end
+    state = next_state;
+  end
+endfunction
+
+
+% reshape matrix m as a vector v by reading out elements in zig-zag pattern
+
+function v = mtov_zigzag(m)
+  [nr nc] = size(m);
+  map = zigzag(nr,nc)
+  v = zeros(1,nr*nc);
+  for r=1:nr
+    for c=1:nc
+      v(map(r,c)) = m(r,c);
+    end
+  end
+endfunction
+
+
+% extracts DCT information for rate K surface
+
+function unwrapped_dcts = dct_blocks(surf, Nt=16)
+  [frames K] = size(surf);
+
+  % break into 160ms blocks, 2D DCT, truncate, IDCT
+
+  Nblocks = floor(frames/Nt);
+  unwrapped_dcts = zeros(Nblocks,Nt*K);
+
+  for n=1:Nblocks
+    st = (n-1)*Nt+1; en = st + Nt - 1;
+    D = dct2(surf(st:en,:));
+    unwrapped_dcts(n,:) = reshape(D',1,Nt*K);
+  end
+endfunction
+
+
+% Determines a map for quantising 2D DCT coeffs in order of rms value
+% (ie std dev) of each coeff.  Those coeffs with the greatest change
+% need the most bits to quantise
+
+function [map rms_map mx mx_ind unwrapped_dcts] = create_map_rms(rate_K_surface, nr, nc)
+  unwrapped_dcts = dct_blocks(rate_K_surface, nr);
+  [mx mx_ind] = sort(std(unwrapped_dcts));
+  mx_ind = fliplr(mx_ind); mx = fliplr(mx);
+  map = rms_map = zeros(nr,nc);
+  for i=1:nr*nc
+    r = floor((mx_ind(i)-1)/nc) + 1;
+    c = mx_ind(i) - (r-1)*nc;
+    %printf("%d %d %d\n", i, r, c);
+    map(r,c) = i;
+    rms_map(r,c) = mx(i);
+  end
+endfunction
+
+
+% plot histogram of each 2D DCT coeff, so we can get a feel for
+% quantiser design
+
+function plot_dct2_hists(rate_K_surface, nr, nc)
+  [map rms_map mx mx_ind unwrapped_dcts] = create_map_rms(rate_K_surface, nr, nc);
+  Ncoeff = nr*nc;
+  fign = 1; subplotn = 1;
+  close all; figure(fign); clf;
+  Nplot = 60;
+  for i=1:Nplot   
+    subplot(5,4,subplotn);
+    d = unwrapped_dcts(:,mx_ind(i));
+    d = round(d/4);
+    hist(d);
+    subplotn++;
+    if (subplotn > 20) && (i != Nplot)
+      subplotn = 1;
+      fign++;
+      figure(fign); clf;
+    end
+  end
+endfunction
+
+
+% Gather run length data for each 2D DCT coeff, to see if run length encoding
+% can help
+
+function [run_length d]= plot_run_length(rate_K_surface, nr, nc)
+  [map rms_map mx mx_ind unwrapped_dcts] = create_map_rms(rate_K_surface, nr, nc);
+  Ncoeff = nr*nc;
+  [Nblocks tmp] = size(unwrapped_dcts);
+
+  % first get histogram of DCT values -----------------------------------
+
+  % some mild quantisation
+
+  unwrapped_dcts = round(unwrapped_dcts/4);
+
+  % note we only consider first half of DCT coeffs, unlikely to use all 
+
+  d = [];
+  for i=1:Nblocks
+    d = [d unwrapped_dcts(i,mx_ind(1:Ncoeff/2))]; 
+  end
+
+  % note we remove outliers from plot as very low prob symbols
+
+  d = d(find(abs(d)<10));
+  figure(1); clf; [Wi, ii] = hist(d,-10:10,1); plot(ii,Wi);
+  length(d)
+  Wi = Wi(find(Wi > 0));
+  %sum(Wi)
+  %-log2(Wi)
+  %-Wi .* log2(Wi)
+  printf("bits/symbol: %2.2f\n", sum(-Wi .* log2(Wi)));
+
+  % now measure run lengths --------------------------------------------
+  
+  run_length = zeros(21,Ncoeff);
+  state = 'idle';
+
+  for i=2:length(d)
+
+    next_state = state;
+
+    if state == 'idle'
+      if d(i-1) == d(i)
+        next_state = 'trac';
+        arun_length = 2;
+      else
+        run_length(d(i)+10, 1)++;
+      end
+    end
+
+    if state == 'trac'
+      if d(i-1) == d(i)
+        arun_length++;
+      else
+        next_state = 'idle';
+        run_length(d(i-1)+10, arun_length)++;
+      end
+    end
+
+    state = next_state;
+
+  end
+
+  figure(2); clf; mesh(run_length(:,1:10));
+endfunction
+
+
+% Design kmeans quantisers for each DCT coeff.  This didn't work very well.
+
+function [quantisers nbits] = design_quantisters_kmeans(rate_K_surface, nr, nc, nbits_max)
+  [map rms_map mx unwrapped_dcts] = create_map_rms(rate_K_surface, nr, nc);
+  nq = nr*nc;
+  quantisers = zeros(nq, 2^nbits_max); nbits = zeros(nq,1);
+  for i=1:nq
+
+    % work out number of levels for this quantiser such that it is a
+    % power of 2 for integer number of bits
+
+    nlevels = (2^nbits_max);
+    nbits(i) = round(log2(nlevels));
+    nlevels = 2 .^ nbits(i);
+
+    if i <= 100
+      printf("%d %d\n", i, nlevels);
+      [idx, centers] = kmeans(unwrapped_dcts(:,i), nlevels);
+      quantisers(i,1:nlevels) = sort(centers);
+    end
+  end
+endfunction
+
+
+% Uniform quantisers designed to fit limits of each DCT coeff
+
+function [quantisers nlevels] = design_quantisters_uniform(rate_K_surface, nr, nc, nlevels_max)
+  [map rms_map mx mx_ind unwrapped_dcts] = create_map_rms(rate_K_surface, nr, nc);
+  
+  nq = nr*nc;
+  quantisers = zeros(nq, nlevels_max); nlevels = zeros(nq, 1);
+
+  for i=1:nq
+    d = unwrapped_dcts(:,mx_ind(i));
+    d = floor(d/16);
+    q_min = floor(min(d));
+    q_max = ceil(max(d));
+    nlevels(i) = q_max-q_min+1;
+    quantisers(i,1:nlevels(i)) = 16*(q_min:q_max);
+  end
+endfunction
+
+
 % Determine a phase spectra from a magnitude spectra
 % from http://www.dsprelated.com/showcode/20.php
 % Haven't _quite_ figured out how this works but have to start somewhere ....
 %
 % TODO: we may be able to sample at a lower rate, like mWo
 %       but start with something that works
-
 function [phase Gdbfk s Aw] = determine_phase(model, f, Nfft=512, ak)
   Fs      = 8000;
   max_amp = 80;
@@ -147,7 +445,7 @@ function [phase Gdbfk s Aw] = determine_phase(model, f, Nfft=512, ak)
   Am = model(f,3:(L+2));
   AmdB = 20*log10(Am);
   rate_L_sample_freqs_kHz = (1:L)*Wo*4/pi;
-  Gdbfk = interp_para(rate_L_sample_freqs_kHz, AmdB, sample_freqs_kHz);    
+  Gdbfk = interp_para(rate_L_sample_freqs_kHz, AmdB, Fs/(2*1000), sample_freqs_kHz);    
 
   % Gdbfk = resample_mask(model, f, mask_sample_freqs_kHz);
 
@@ -171,8 +469,8 @@ function mel = ftomel(fHz)
 endfunction
 
 
-function rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K)
-  mel_start = ftomel(200); mel_end = ftomel(3700); 
+function rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K, fstart_hz=100, fend_hz=0.95*4000)
+  mel_start = ftomel(fstart_hz); mel_end = ftomel(fend_hz); 
   step = (mel_end-mel_start)/(K-1);
   mel = mel_start:step:mel_end;
   rate_K_sample_freqs_Hz = 700*((10 .^ (mel/2595)) - 1);
@@ -180,20 +478,26 @@ function rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K)
 endfunction
 
 
-function [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model, K) 
-  rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K);
-  rate_K_surface = resample_const_rate_f(model, rate_K_sample_freqs_kHz);
+function plot_mel_sample_freqs(K, f_start_hz, f_end_hz)
+  rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K, f_start_hz, f_end_hz);
+  figure(1); clf;
+  plot(rate_K_sample_freqs_kHz,'+');
+endfunction
+
+function [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model, K, Fs=8000) 
+  rate_K_sample_freqs_kHz = mel_sample_freqs_kHz(K, 100, 0.95*Fs/2);
+  rate_K_surface = resample_const_rate_f(model, rate_K_sample_freqs_kHz, Fs);
 endfunction
 
 
 % Resample Am from time-varying rate L=floor(pi/Wo) to fixed rate K.  This can be viewed
 % as a 3D surface with time, freq, and ampitude axis.
 
-function [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f(model, rate_K_sample_freqs_kHz)
+function [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f(model, rate_K_sample_freqs_kHz, Fs)
 
   % convert rate L=pi/Wo amplitude samples to fixed rate K
 
-  max_amp = 80;
+  max_amp = 160;
   [frames col] = size(model);
   K = length(rate_K_sample_freqs_kHz);
   rate_K_surface = zeros(frames, K);
@@ -203,48 +507,121 @@ function [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f(model,
     L = min([model(f,2) max_amp-1]);
     Am = model(f,3:(L+2));
     AmdB = 20*log10(Am);
-    %pre = 10*log10((1:L)*Wo*4/(pi*0.3));    
-    %AmdB += pre;
 
-    % clip between peak and peak -50dB, to reduce dynamic range
+    clip_en = 0;
+    if clip_en
+      % clip between peak and peak -50dB, to reduce dynamic range
 
-    AmdB_peak = max(AmdB);
-    AmdB(find(AmdB < (AmdB_peak-50))) = AmdB_peak-50;
+      AmdB_peak = max(AmdB);
+      AmdB(find(AmdB < (AmdB_peak-50))) = AmdB_peak-50;
+    end
+
+    rate_L_sample_freqs_kHz = (1:L)*Wo*Fs/(2000*pi);
     
-    rate_L_sample_freqs_kHz = (1:L)*Wo*4/pi;
-    
-    %rate_K_surface(f,:) = interp1(rate_L_sample_freqs_kHz, AmdB, rate_K_sample_freqs_kHz, "spline", "extrap");
-    rate_K_surface(f,:)  = interp_para(rate_L_sample_freqs_kHz, AmdB, rate_K_sample_freqs_kHz);    
+    rate_K_surface(f,:) = interp1([0 rate_L_sample_freqs_kHz (Fs/2000)], [AmdB(1) AmdB AmdB(L)], rate_K_sample_freqs_kHz, "spline");
+    %rate_K_surface(f,:)  = interp_para(rate_L_sample_freqs_kHz, AmdB, Fs/(2*1000), rate_K_sample_freqs_kHz);    
 
-    %printf("\r%d/%d", f, frames);
+    %printf("%d\n", f);
   end
   %printf("\n");
 endfunction
 
 
+function [rate_K_vec_corrected orig_error error nasty_error_log nasty_error_m_log] = correct_rate_K_vec(rate_K_vec, rate_K_sample_freqs_kHz, AmdB, AmdB_, K, Wo, L, Fs)
+
+    % aliasing correction --------------------------------------
+
+    % The mel sample rate decreases as frequency increases. Look for
+    % any regions above 1000Hz where we have missed definition of a
+    % spectral peak (formant) due to aliasing.  Adjust the rate K
+    % sample levels to restore peaks.  Theory is that correct
+    % definition of a formant is less important than the frequency of
+    % the formant.  As long as we define a formant in that general
+    % frequency area it will sound OK.
+
+    Am_freqs_kHz = (1:L)*Wo*4/pi;
+
+    % Lets see where we have made an error
+
+    error = orig_error = AmdB - AmdB_;
+
+    Ncorrections = 3;      % maximum number of rate K samples to correct
+    error_thresh = 3;      % only worry about errors larger than thresh
+
+    start_m = floor(L*1000/(Fs/2));
+    error(1:start_m) = 0;  % first 1000Hz is densly sampled so ignore
+    nasty_error_m_log = []; nasty_error_log = [];
+
+
+    rate_K_vec_corrected = rate_K_vec;
+    for i=1:Ncorrections
+      [mx mx_m] = max(error);
+
+      if mx > error_thresh
+        nasty_error_log = [nasty_error_log mx];
+        nasty_error_m_log = [nasty_error_m_log mx_m];
+
+        % find closest rate K sample to nasty error
+
+        nasty_error_freq = mx_m*Wo*Fs/(2*pi*1000);
+        [tmp closest_k] = min(abs(rate_K_sample_freqs_kHz - nasty_error_freq));
+        rate_K_vec_corrected(closest_k) = AmdB(mx_m);
+
+        % zero out error in this region and look for another large error region
+
+        k = max(1, closest_k-1); 
+        rate_K_prev_sample_kHz = rate_K_sample_freqs_kHz(k);
+        k = min(K, closest_k+1); 
+        rate_K_next_sample_kHz = rate_K_sample_freqs_kHz(k);
+
+        [tmp st_m] = min(abs(Am_freqs_kHz - rate_K_prev_sample_kHz));
+        [tmp en_m] = min(abs(Am_freqs_kHz - rate_K_next_sample_kHz));
+        if closest_k == K
+         en_m = L;
+        end 
+        error(st_m:en_m) = 0;
+      end
+    end
+endfunction
+
+
 % Take a rate K surface and convert back to time varying rate L
 
-function [model_ AmdB_] = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz)
-  max_amp = 80;
+function [model_ AmdB_] = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz, Fs=8000, pad_end=0)
+  max_amp = 160; K = columns(rate_K_surface);
   [frames col] = size(model);
 
+  AmdB_ = zeros(frames, max_amp);
   model_ = zeros(frames, max_amp+2);
   for f=1:frames
     Wo = model(f,1);
     L = model(f,2);
-    rate_L_sample_freqs_kHz = (1:L)*Wo*4/pi;
+    rate_L_sample_freqs_kHz = (1:L)*Wo*Fs/(2000*pi);
     
     % back down to rate L
 
-    % AmdB_ = interp1(rate_K_sample_freqs_kHz, rate_K_surface(f,:), rate_L_sample_freqs_kHz, "spline", 0);
-    AmdB_ = interp_para([ 0 rate_K_sample_freqs_kHz 4], [0 rate_K_surface(f,:) 0], rate_L_sample_freqs_kHz);
+    % dealing with end effects is an ongoing issue.....need a better solution
 
-    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB_(1:L)/20);
+    if pad_end
+      AmdB_(f,1:L) = interp1([0 rate_K_sample_freqs_kHz Fs/2000], 
+                             [rate_K_surface(f,1) rate_K_surface(f,:) rate_K_surface(f,K)], 
+                             rate_L_sample_freqs_kHz, 
+                             "spline");
+    else
+      AmdB_(f,1:L) = interp1([0 rate_K_sample_freqs_kHz], 
+                             [rate_K_surface(f,1) rate_K_surface(f,:)], 
+                             rate_L_sample_freqs_kHz, 
+                             "spline");
+    end
+
+    %AmdB_(f,1:L) = interp_para(rate_K_sample_freqs_kHz, rate_K_surface(f,:), Fs/(2*1000), rate_L_sample_freqs_kHz);
+    %printf("f: %d %f %f %f\n", f, rate_K_sample_freqs_kHz(1), rate_L_sample_freqs_kHz(1), AmdB_(1));
+    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB_(f, 1:L)/20);
    end
 endfunction
 
 
-% Post Filter, has a big impact on speech quality after VQ.  When used
+% PostFilter, has a big impact on speech quality after VQ.  When used
 % on a mean removed rate K vector, it raises formants, and supresses
 % anti-formants.  As it manipulates amplitudes, we normalise energy to
 % prevent clipping or large level variations.  pf_gain of 1.2 to 1.5
@@ -350,6 +727,43 @@ function [Wo_ voicing_] = interp_Wo_v(Wo1, Wo2, voicing1, voicing2)
 endfunction
 
 
+function [diff_weighted weights error g min_ind] = search_vq_weighted(target, vq, weight_gain)
+  [vq_rows vq_cols] = size(vq);
+
+  weight_gain = 0.1; % I like this vairable name as it is funny
+
+  % find mse for each vector
+
+  error = g = zeros(1, vq_rows);
+  diff = weights = diff_weighted = zeros(vq_rows, vq_cols);
+
+  weights = max(0.1, weight_gain .* (target + 20));
+
+  for i=1:vq_rows
+
+    % work out gain for best match
+
+    g(i) = sum((target - vq(i,:)).*weights)/vq_cols;
+
+    % Find weighted difference.  This allocated more importance
+    % (error) to samples with higher energy, and stops really low
+    % level harmonics from having any impact.  Note addition in dB
+    % is multiplication in linear
+
+    diff(i,:) = target - vq(i,:) - g(i);
+ 
+    diff_weighted(i,:) = diff(i,:) .* weights;
+
+    % abs in dB is MSE in linear
+
+    error(i) = mean(abs(diff_weighted(i,:)));
+  end
+
+  [mn min_ind] = min(error);
+  
+endfunction
+
+
 % --------------------------------------------------------------------------------
 % Experimental functions used for masking, piecewise models, not part of newamp1
 % --------------------------------------------------------------------------------
@@ -445,24 +859,24 @@ endfunction
 % determine cumulative mask, using amplitude of each harmonic.  Mask is
 % sampled across L points in the linear domain
 
-function maskdB = determine_mask(masker_amps_dB, masker_freqs_kHz, mask_sample_freqs_kHz)
+function maskdB = determine_mask(masker_amps_dB, masker_freqs_kHz, mask_sample_freqs_kHz, bark_model=1)
 
     % calculate and plot masking curve
 
-    maskdB = zeros(1,length(mask_sample_freqs_kHz));
+    maskdB = -20*ones(1,length(mask_sample_freqs_kHz));
     for m=1:length(masker_freqs_kHz)
-      %maskdB = max(maskdB, schroeder(masker_freqs_kHz(m), mask_sample_freqs_kHz) + masker_amps_dB(m)); 
-      maskdB = max(maskdB, parabolic_resonator(masker_freqs_kHz(m), mask_sample_freqs_kHz) + masker_amps_dB(m)); 
+      maskdB = max(maskdB, schroeder(masker_freqs_kHz(m), mask_sample_freqs_kHz, bark_model) + masker_amps_dB(m)); 
+      %maskdB = max(maskdB, parabolic_resonator(masker_freqs_kHz(m), mask_sample_freqs_kHz) + masker_amps_dB(m)); 
     end
 end
 
 
 % Sample mask as model for Am
 
-function [maskdB Am_freqs_kHz] = mask_model(AmdB, Wo, L)
+function [maskdB Am_freqs_kHz] = mask_model(AmdB, Wo, L, bark_model=1)
 
     Am_freqs_kHz = (1:L)*Wo*4/pi;
-    maskdB = determine_mask(AmdB, Am_freqs_kHz, Am_freqs_kHz);
+    maskdB = determine_mask(AmdB, Am_freqs_kHz, Am_freqs_kHz, bark_model);
 endfunction
 
 
@@ -684,30 +1098,30 @@ endfunction
 
 % plot some masking curves, used for working on masking filter changes
 
-function plot_masking
+function plot_masking(bark_model=0);
   Fs = 8000;
 
   figure(1)
   mask_sample_freqs_kHz = 0.1:0.025:(Fs/1000)/2;
-  #{
-  maskdB_s0 = schroeder(0.5, mask_sample_freqs_kHz, 0);
-  plot(mask_sample_freqs_kHz, maskdB_s0,';schroeder 0;');
-  maskdB_s1 = schroeder(0.5, mask_sample_freqs_kHz, 1);
+  %maskdB_s0 = schroeder(0.5, mask_sample_freqs_kHz, 0);
+  %plot(mask_sample_freqs_kHz, maskdB_s0,';schroeder 0;');
+  maskdB_s1 = schroeder(0.5, mask_sample_freqs_kHz, bark_model);
   plot(mask_sample_freqs_kHz, maskdB_s1,'g;schroeder 1;');
-  #}
+  #{
   maskdB_res = parabolic_resonator(0.5, mask_sample_freqs_kHz);
   plot(mask_sample_freqs_kHz, maskdB_res,'r;resonator;');
+  #}
   hold on;
 
   for f=0.5:0.5:3
-    #{
-    maskdB_s0 = schroeder(f, mask_sample_freqs_kHz, 0);
-    plot(mask_sample_freqs_kHz, maskdB_s0);
-    maskdB_s1 = schroeder(f, mask_sample_freqs_kHz, 1);
+    %maskdB_s0 = schroeder(f, mask_sample_freqs_kHz, 0);
+    %plot(mask_sample_freqs_kHz, maskdB_s0);
+    maskdB_s1 = schroeder(f, mask_sample_freqs_kHz, bark_model);
     plot(mask_sample_freqs_kHz, maskdB_s1,'g');
-    #}
+    #{
     maskdB_res = parabolic_resonator(f, mask_sample_freqs_kHz);
     plot(mask_sample_freqs_kHz, maskdB_res,'r;resonator;');
+    #}
   end
   hold off;
 
@@ -1190,7 +1604,7 @@ function [AmdB_ residual fvec fvec_ amps] = piecewise_model(AmdB, Wo, vq, vq_m)
     plot(mask_sample_freqs_kHz, residual);
     end
 
-    %printf("\nfr1: %f fr2: %f fr3: %f fr4: %f\n", fr1, fr2, fr3, fr4);
+    printf("\nfr1: %f fr2: %f fr3: %f fr4: %f\n", fr1, fr2, fr3, fr4);
     [fvec fvec_ind] = sort([fr1 fr2 fr3 fr4]);
     amps = amp(fvec_ind(1:4));
 
@@ -1266,3 +1680,12 @@ function lmin = abys(AmdB_, AmdB, Wo, L, mask_sample_freqs_kHz)
 endfunction
 
 
+function rate_K_surface_no_slope = remove_slope(rate_K_surface)
+  [frames K] = size(rate_K_surface);
+  rate_K_surface_no_slope = zeros(frames,K);
+  for f=1:frames
+    [gradient intercept] = linreg(1:K, rate_K_surface(f,:), K);
+    printf("f: %d gradient: %f intercept: %f\n", f, gradient, intercept);
+    rate_K_surface_no_slope(f,:) = rate_K_surface(f,:) - (intercept + gradient*(1:K));
+  end
+endfunction
