@@ -7,7 +7,7 @@
 #{
 
   Octave script to batch process model parameters using the new
-  amplitude model, version 1.  Outputs anotehr set of model parameters
+  amplitude model, version 1.  Outputs another set of model parameters
   that can be fed to c2sim for listening tests.  The companion
   newamp1_fbf.m script is used to visualise the processing frame by frame
  
@@ -31,8 +31,8 @@
   DSP.  We switch on lpc 10 just to dump voicing.
 
     $ ./c2sim ../../raw/vk5qi.raw --phase0 --postfilter --dump vk5qi --lpc 10 --dump_pitch_e vk5qi_pitche.txt
-    octave:14> newamp1_batch("../build_linux/src/vk5qi", "../build_linux/src/vk5qi_am_Wo.out");
-    $ ./c2sim ../../raw/vk5qi.raw --phase0 --postfilter --amread vk5qi_am_Wo.out --awread vk5qi_aw.out --Woread vk5qi_Wo.out --hand_voicing vk5qi_v.txt -o - | play -q -t raw -r 8000 -s -2 -
+    octave:14> newamp1_batch("../build_linux/src/vk5qi", "../build_linux/src/vk5qi.out");
+    $ ./c2sim ../../raw/vk5qi.raw --phase0 --postfilter --amread vk5qi_am.out --hmread vk5qi_hm.out --Woread vk5qi_Wo.out --hand_voicing vk5qi_v.txt -o - | play -q -t raw -r 8000 -s -2 -
 
 #}
 
@@ -40,17 +40,71 @@
 % In general, this function processes a bunch of amplitudes, we then
 % use c2sim to hear the results.  Bunch of different experiments below
 
-function surface = newamp1_batch(input_prefix, output_prefix)
+function [surface mean_f] = newamp1_batch(input_prefix, varargin)
   newamp;
   more off;
 
-  max_amp = 80;
-  postfilter = 0;   % optional postfiler that runs on Am, not used atm
-  synth_phase = 1;
+  max_amp = 160;
+  mean_f = [];
 
-  if nargin == 1
-    output_prefix = input_prefix;
+  % defaults
+
+  synth_phase = output = 1;
+  output_prefix = input_prefix;
+  vq_type = "";
+  vq_filename = ""; 
+  vq_search = "mse";
+  mode = "const";
+  fit_order = 0;
+
+  % parse variable argument list
+
+  if (length (varargin) > 0)
+
+    % check for the "output_prefix" option
+
+    ind = arg_exists(varargin, "output_prefix");
+    if ind
+      output_prefix =  varargin{ind+1};
+    end
+    ind = arg_exists(varargin, "mode");
+    if ind
+      mode =  varargin{ind+1};
+    end
+    ind = arg_exists(varargin, "vq_search");
+    if ind
+      vq_search = varargin{ind+1};
+    end
+    ind = arg_exists(varargin, "vq");
+    if ind
+      vq_type = varargin{ind};
+      vq_filename = varargin{ind+1};
+      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
+    end
+    ind = arg_exists(varargin, "vqh");
+    if ind
+      vq_type = varargin{ind};
+      vq_filename = varargin{ind+1};
+      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
+    end
+    ind = arg_exists(varargin, "vql");
+    if ind
+      vq_type = varargin{ind};
+      vq_filename = varargin{ind+1};
+      printf("vq_type: %s vq_filename: %s vq_search: %s\n", vq_type, vq_filename, vq_search);
+    end
+    ind = arg_exists(varargin, "no_output");
+    if ind
+      output = 0;
+      synth_phase = 0;
+    end
   end
+
+  printf("output: %d\n", output);
+  if (output)
+    printf("output_prefix: %s\n",  output_prefix);
+  end
+  
   model_name = strcat(input_prefix,"_model.txt");
   model = load(model_name);
   [frames nc] = size(model);
@@ -65,97 +119,87 @@ function surface = newamp1_batch(input_prefix, output_prefix)
 
   % Choose experiment to run test here -----------------------
 
-  %model_ = experiment_filter(model);
-  %model_ = experiment_filter_dec_filter(model);
-
-  %[model_ surface] = experiment_mel_freq(model, 1, 1, voicing);
-  %model_ = experiment_dec_abys(model, 8, 1, 1, 1, voicing);
-
-  [model_ voicing_ indexes] = experiment_rate_K_dec(model, voicing); % encoder/decoder, lets toss away results except for indexes
-  %[model_ voicing_] = model_from_indexes(indexes);                   % decoder uses just indexes, outputs vecs for synthesis
-  [model_ voicing_] = model_from_indexes_fbf(indexes);                   % decoder uses just indexes, outputs vecs for synthesis
-
-  %model_ = experiment_dec_linear(model_);
-  %model_ = experiment_energy_rate_linear(model, 1, 0);
-
-  %[model_ surface] = experiment_mel_diff_freq(model, 0);
-  %[model_ rate_K_surface] = experiment_closed_loop_mean(model);
+  if strcmp(mode, 'dct2')
+    [model_ surface] = experiment_rate_K_dct2(model, 0, 1, voicing);
+  end
+  if strcmp(mode, 'mel')
+    [model_ surface] = experiment_mel_freq(model, 0, 1, voicing);
+  end
+  if strcmp(mode, 'const')
+    [model_ surface] = experiment_const_freq(model, vq_type, vq_filename, vq_search);
+  end
+  if strcmp(mode, 'piecewise')
+    model_ = experiment_piecewise(model);
+  end
+  if strcmp(mode, 'dct')
+    model_ = experiment_dct(model);
+  end
 
   % ----------------------------------------------------
 
-  Am_out_name = sprintf("%s_am.out", output_prefix);
-  fam  = fopen(Am_out_name,"wb"); 
-
-  Wo_out_name = sprintf("%s_Wo.out", output_prefix);
-  fWo  = fopen(Wo_out_name,"wb"); 
+  if output
+    Am_out_name = sprintf("%s_am.out", output_prefix);
+    fam  = fopen(Am_out_name,"wb"); 
+ 
+    Wo_out_name = sprintf("%s_Wo.out", output_prefix);
+    fWo  = fopen(Wo_out_name,"wb"); 
   
-  if synth_phase
-    Hm_out_name = sprintf("%s_hm.out", output_prefix);
-    fhm = fopen(Hm_out_name,"wb"); 
+    if synth_phase
+      Hm_out_name = sprintf("%s_hm.out", output_prefix);
+      fhm = fopen(Hm_out_name,"wb"); 
+    end
+
+    for f=1:frames
+      %printf("%d ", f);   
+      Wo = model_(f,1); L = min([model_(f,2) max_amp-1]); Am = model_(f,3:(L+2));
+      if Wo*L > pi
+        printf("Problem: %d  Wo*L > pi\n", f);   
+      end
+
+      Am_ = zeros(1,max_amp); Am_(2:L) = Am(1:L-1); fwrite(fam, Am_, "float32");
+      fwrite(fWo, Wo, "float32");
+
+      if synth_phase
+
+        % synthesis phase spectra from magnitiude spectra using minimum phase techniques
+
+        fft_enc = 256;
+        phase = determine_phase(model_, f, fft_enc);
+        assert(length(phase) == fft_enc);
+
+        % sample phase at centre of each harmonic, not 1st entry Hm[1] in octave Hm[0] in C
+        % is not used
+
+        Hm = zeros(1, 2*max_amp);
+        for m=1:L
+          b = round(m*Wo*fft_enc/(2*pi));
+          Hm(2*m) = cos(phase(b));
+          Hm(2*m+1) = -sin(phase(b));
+        end
+        fwrite(fhm, Hm, "float32");    
+      end
+    end
+ 
+   fclose(fam);
+   fclose(fWo);
+   if synth_phase
+     fclose(fhm);
+   end
+
+   % save voicing file
+  
+   if exist("voicing_", "var")
+     v_out_name = sprintf("%s_v.txt", output_prefix);
+     fv  = fopen(v_out_name,"wt"); 
+     for f=1:length(voicing_)
+       fprintf(fv,"%d\n", voicing_(f));
+     end
+     fclose(fv);
+    end
   end
 
   for f=1:frames
-    printf("%d ", f);   
-    Wo = model_(f,1);
-    L = min([model_(f,2) max_amp-1]);
-    Am = model_(f,3:(L+2));
-
-    Am_ = zeros(1,max_amp);
-    Am_(2:L) = Am(1:L-1);
-
-    % optional post filter on linear {Am}, boosts higher amplitudes more than lower,
-    % improving shape of formants and reducing muffling.  Note energy
-    % normalisation
-
-    if postfilter
-      e1 = sum(Am_(2:L).^2);
-      Am_(2:L) = Am_(2:L) .^ 1.5;         
-      e2 = sum(Am_(2:L).^2);
-      Am_(2:L) *= sqrt(e1/e2);
-    end
-
-    fwrite(fam, Am_, "float32");
-    fwrite(fWo, Wo, "float32");
-
-    if synth_phase
-
-      % synthesis phase spectra from magnitiude spectra using minimum phase techniques
-
-      fft_enc = 128;
-      phase = determine_phase(model_, f, fft_enc);
-      assert(length(phase) == fft_enc);
-      %Aw = zeros(1, fft_enc*2); 
-      %Aw(1:2:fft_enc*2) = cos(phase);
-      %Aw(2:2:fft_enc*2) = -sin(phase);
-
-      % sample phase at centre of each harmonic, not 1st entry Hm[1] in octave Hm[0] in C
-      % is not used
-
-      Hm = zeros(1, 2*max_amp);
-      for m=1:L
-        b = round(m*Wo*fft_enc/(2*pi));
-        Hm(2*m) = cos(phase(b));
-        Hm(2*m+1) = -sin(phase(b));
-      end
-      fwrite(fhm, Hm, "float32");    
-    end
-  end
- 
-  fclose(fam);
-  fclose(fWo);
-  if synth_phase
-    fclose(fhm);
-  end
-
-  % save voicing file
-  
-  if exist("voicing_", "var")
-    v_out_name = sprintf("%s_v.txt", output_prefix);
-    fv  = fopen(v_out_name,"wt"); 
-    for f=1:length(voicing_)
-      fprintf(fv,"%d\n", voicing_(f));
-    end
-    fclose(fv);
+    surface(f,:) -= mean(surface(f,:));
   end
 
   printf("\n")
@@ -163,6 +207,162 @@ function surface = newamp1_batch(input_prefix, output_prefix)
 endfunction
  
 
+function surface = slope_and_mean_removal(surface)
+  [frames K] = size(surface);
+  for f=1:frames
+    v = surface(f,:);
+    [m b] = linreg(1:K, v, K);
+    v -= m*(1:K) + b;
+    surface(f,:) = v;
+  end
+endfunction
+
+
+function ind = arg_exists(v, str) 
+   ind = 0;
+   for i=1:length(v)
+      if strcmp(v{i}, str)
+        ind = i;
+      end     
+    end
+endfunction
+
+
+% Basic unquantised rate K linear sampling then back to rate L.  Used for generating 
+% training vectors and testing vector quntisers.
+
+function [model_ rate_K_surface] = experiment_const_freq(model, vq_type, vq_filename, vq_search="mse")
+  melvq;
+  [frames nc] = size(model);
+  Fs = 8000;
+  fg = 1;
+  quant_en = 0;
+
+  rate_K_sample_freqs_kHz = [0.1:0.1:4];
+  K = length(rate_K_sample_freqs_kHz);
+
+  % optional full band VQ
+
+  if strcmp(vq_type, "vq")
+    quant_en = 1;
+    x = load(vq_filename); vq = x.vq;
+    [vq_rows vq_cols] = size(vq); vq_st = 1; vq_en = vq_cols;
+  end
+ 
+  if strcmp(vq_type, "vql")
+    quant_en = 1;
+    x = load(vq_filename); vq = x.vq;
+    [vq_rows vq_cols] = size(vq); vq_st = 1; vq_en = vq_st + vq_cols - 1;
+  end
+
+  if strcmp(vq_type, "vqh")
+    quant_en = 1;
+    x = load(vq_filename); vq = x.vq;
+    [vq_rows vq_cols] = size(vq); vq_st = 11; vq_en = K;
+  end
+
+  energy = zeros(1,frames);
+  for f=1:frames
+    L = model(f,2);
+    energy(f) = 10*log10(sum( model(f,3:(L+2)) .^ 2 ));
+  end
+
+  rate_K_surface = resample_const_rate_f(model, rate_K_sample_freqs_kHz, Fs);
+
+  rate_K_surface_fit = zeros(frames,K);
+  b = zeros(1,frames);
+
+  for f=1:frames
+    b(f) = mean(rate_K_surface(f,:));
+    rate_K_surface_fit(f,:) = rate_K_surface(f,:) - b(f);
+  end
+ 
+  % optional vector quantise
+  
+  if quant_en
+ 
+    rate_K_surface_fit_ = rate_K_surface_fit;
+    res = zeros(frames, vq_cols); ind = [];
+
+    for f=1:frames
+      target = rate_K_surface_fit(f, vq_st:vq_en);
+
+      if strcmp(vq_search, "mse")
+        [idx contrib errors test_ g mg sl] = vq_search_mse(vq, target);
+        rate_K_surface_fit_(f, vq_st:vq_en) = contrib;
+      end
+
+      if strcmp(vq_search, "slope")
+        [idx contrib errors test_ g mg sl] = vq_search_slope(vq, target);
+        rate_K_surface_fit_(f, vq_st:vq_en) = contrib;
+      end
+      
+      res(f, vq_st:vq_en) = target - contrib;
+      ind = [ind idx];
+    end
+    
+    figure(fg++); clf; mesh(res);
+    sd_per_frame = std(res(:,vq_st:vq_en)');
+    figure(fg++); subplot(211); plot(energy); subplot(212); plot(sd_per_frame);
+    figure(fg++); subplot(211); hist(sd_per_frame); subplot(212); hist(ind,100);
+    printf("VQ rms SD: %3.2f\n", mean(sd_per_frame));
+  else
+    rate_K_surface_fit_ = rate_K_surface_fit;
+  end
+
+  for f=1:frames
+    rate_K_surface_(f,:) = rate_K_surface_fit_(f,:) + b(f);
+  end
+
+  [model_ AmdB_] = resample_rate_L(model, rate_K_surface_, rate_K_sample_freqs_kHz, Fs);
+
+  % Measure distortion between AmdB and AmdB_, ths includes distortion
+  % in the rate K <-> L transition.  Can optionally plot distorted
+  % frames
+
+  plot_sd_thresh = 5;
+  sd = zeros(1,frames);
+  for f=1:frames
+    Wo = model(f,1);
+    L = model(f,2);
+    AmdB = 20*log10(model(f,3:(L+2)));
+    sd(f) = std(AmdB(1:L) - AmdB_(f,1:L));
+    if (sd(f) > plot_sd_thresh) && (fg < 10)
+      printf("fg: %d f: %d\n", fg, f);
+      figure(fg++); clf; plot((1:L)*Wo*4/pi, AmdB(1:L),'b+-'); hold on; plot((1:L)*Wo*4/pi, AmdB_(f,1:L),'r+-');
+      plot(rate_K_sample_freqs_kHz, rate_K_surface_(f,:), 'c+-'); hold off;
+     end
+  end
+  printf("rate K resampling SD: %3.2f\n", mean(sd));
+  figure(fg++); clf; subplot(211); plot(energy); subplot(212); plot(sd); title('sdL');
+  figure(fg++); clf; hist(sd);
+endfunction
+
+
+function model_ = experiment_dct(model)
+
+  [frames tmp] = size(model); max_amp = 160;
+
+  for f=1:frames
+    printf("%d ", f);   
+    Wo = model(f,1);
+    L = min([model(f,2) max_amp-1]);
+    Am = model(f,3:(L+2));
+    AmdB = 20*log10(Am);
+
+    % fit model
+
+    D = dct(AmdB);
+    E = zeros(1,L);
+    E(1:min(20,L)) = D(1:min(20,L));
+    AmdB_ = idct(D);
+
+    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB_(1:L)/20);
+  end
+
+endfunction
+
+
 % -----------------------------------------------------------------------------------------
 % Linear decimator/interpolator that operates at rate K, includes VQ, post filter, and Wo/E
 % quantisation.  Evolved from abys decimator below.  Simulates the entire encoder/decoder.
@@ -484,57 +684,181 @@ endfunction
 
 
 
-
 % ---------------------------------------------------------------------------------------
 % Various experiments tried during development
 
-% experiment to resample freq axis on mel scale, then optionally vq
+% rate K mel-resampling, high end correction, and DCT experiment workhorse
+
+function [model_ rate_K_surface] = experiment_rate_K_dct2(model, vq_en=0, plots=1, voicing)
+  [frames nc] = size(model);
+  K = 20; Fs = 8000; correct_rate_K_en = 1;
+
+  %quantisers = load("dct2quant.txt"); 
+  %nlevels = load("dct2quantnlevels.txt");
+  %map = load("dct2map.txt");
+
+  for f=1:frames
+    Wo = model(f,1);
+    L = model(f,2);
+    Am = model(f,3:(L+2));
+    AmdB = 20*log10(Am);
+    Am_freqs_kHz = (1:L)*Wo*4/pi;
+    [rate_K_vec rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model(f,:), K);
+    if correct_rate_K_en
+      [tmp_ AmdB_] = resample_rate_L(model(f,:), rate_K_vec, rate_K_sample_freqs_kHz);
+      [rate_K_vec_corrected orig_error error nasty_error_log nasty_error_m_log] = correct_rate_K_vec(rate_K_vec, rate_K_sample_freqs_kHz, AmdB, AmdB_, K, Wo, L, Fs);
+      rate_K_surface(f,:) = rate_K_vec_corrected;
+    else
+      rate_K_surface(f,:) = rate_K_vec;
+    end
+  end
+
+  % break into 160ms blocks, 2D DCT, truncate, IDCT
+
+  Tf = 0.01;   % frame period in seconds
+  Nt = 16;     % number of 10ms frames blocks in time
+  dec = 2;     % decimation factor
+  dist_dB = 2; % use enough coefficients to get this distortion ond DCT coeffs
+
+  Nblocks = floor(frames/(Nt*dec));
+  printf("frames: %d Nblocks: %d\n", frames, Nblocks);
+
+  unwrapped_dcts = zeros(Nblocks,Nt*K);
+  rate_K_surface_ = zeros(frames, K);
+  
+  % create map on the fly from train database
+
+  asurf = load("all_surf.txt"); [nr nc] = size(asurf);
+  asurf = asurf(1:dec:nr,:);
+  [map rms_map mx mx_ind unwrapped_dcts] = create_map_rms(asurf, Nt, K);
+  %map = create_zigzag_map(Nt,K);
+
+  %printf("non zero coeffs: %d\n", sum(sum(map == 1)));
+  figure(2); clf;
+  mesh(map);
+  sumnz = zeros(1,Nblocks);
+  dct2_sd = zeros(1,Nblocks);
+
+  % create quantiser_num to r,c Luts
+
+  rmap = cmap = zeros(1,Nt*K);
+  for r=1:Nt
+    for c=1:K
+       quantiser_num = map(r,c);
+       rmap(quantiser_num) = r;
+       cmap(quantiser_num) = c;
+    end
+  end
+
+  for n=1:Nblocks
+    st = (n-1)*dec*Nt+1; en = st + dec*Nt - 1;
+    %printf("st: %d en: %d\n", st, en);
+    D = dct2(rate_K_surface(st:dec:en,:));
+
+    % move over surface and work out quantiser
+    % quantise, replace on map
+
+    E = mapped = zeros(Nt,K);
+    #{
+    for r=1:Nt
+      for c=1:K
+        quantiser_num = map(r,c);
+        if quantiser_num <= 40
+          %printf("r %d c %d quantiser_num %d nlevels %d ", r, c, quantiser_num, nlevels(quantiser_num));
+          %levels = quantisers(quantiser_num, 1:nlevels(quantiser_num));
+          %quant_out = quantise(levels, D(r,c));
+          E(r,c) = D(r,c);
+          if E(r,c)
+            mapped(r,c) = 1;
+            sumnz(n)++;
+          end
+        end
+      end
+    end
+    #}
+
+    qn = 0;
+    adct2_sd = mean(std(D-E));
+    while adct2_sd > dist_dB
+      qn++;
+      E(rmap(qn), cmap(qn)) = 1*round(D(rmap(qn), cmap(qn))/1);
+      adct2_sd = mean(std(D-E));
+      %printf("qn %d %f\n", qn, adct2_sd);
+    end
+    sumnz(n) = qn;
+
+    % note neat trick to interpolate to 10ms frames despite dec
+
+    #{
+    energy = sum(sum(E));
+    Edc = E(1,1);
+    E = E*1.2;
+    E(1,1) = Edc/1.2;
+    #}
+    %E *= energy/sum(sum(E));
+    rate_K_surface_(st:en,:) = idct2([sqrt(dec)*E; zeros(Nt*(dec-1), K)]);
+
+    dct2_sd(n) = mean(std(D-E));
+  end
+
+  % figure(3); clf; mesh(mapped);
+  figure(4); clf; plot(sumnz); hold on; plot([1 length(sumnz)],[mean(sumnz) mean(sumnz)]); hold off; title('Non Zero');
+  figure(5); clf; plot(dct2_sd); title('DCT SD');
+  printf("average dct spectral distortion: %3.2f dB\n", mean(dct2_sd));
+  printf("mean number of coeffs/DCT: %3.2f/%d\n", mean(sumnz), Nt*K);
+  printf("coeffs/second: %3.2f\n", mean(sumnz)/(Nt*Tf*dec));
+  printf("bits/s: %3.2f\n", 2.9*mean(sumnz)/(Nt*Tf*dec));
+
+  % optional 700C style post filter
+
+  post_filter_en = 0;
+  if post_filter_en
+    for f=1:Nt*Nblocks
+      mn = mean(rate_K_surface_(f,:));
+      rate_K_surface_no_mean_(f,:) =  rate_K_surface_(f,:) - mn;
+      rate_K_surface_(f,:) = mn + post_filter(rate_K_surface_no_mean_(f,:), rate_K_sample_freqs_kHz);
+    end
+  end
+
+  % prevent /0 errors at end of run
+
+  rate_K_surface_(dec*Nt*Nblocks+1:frames,:) = rate_K_surface(dec*Nt*Nblocks+1:frames,:); 
+  model_ = resample_rate_L(model, rate_K_surface_, rate_K_sample_freqs_kHz);
+  
+  dist = std((rate_K_surface_(1:dec:frames,:) - rate_K_surface(1:dec:frames,:))');
+  figure(1); clf; plot(dist); title('Rate K SD');
+  printf("Rate K spectral distortion mean: %3.2f dB var: %3.2f\n", mean(dist), var(dist));
+endfunction
+
+
+% Basic unquantised rate K mel-sampling then back to rate L.  Now with "high end correction"
 
 function [model_ rate_K_surface] = experiment_mel_freq(model, vq_en=0, plots=1, voicing)
   [frames nc] = size(model);
-  K = 20; 
-  [rate_K_surface rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model, K);
-  
-  if plots
-    figure(1); clf; mesh(rate_K_surface);
-    figure(2); clf; plot_dft_surface(rate_K_surface)
-  end
+  K = 20; Fs = 16000; correct_rate_K_en = 0;
 
   for f=1:frames
-    mean_f(f) = mean(rate_K_surface(f,:));
-    rate_K_surface_no_mean(f,:) = rate_K_surface(f,:) - mean_f(f);
-  end
-
-  if vq_en
-    melvq;
-    load train_120_vq; m=5;
-       
-    [res rate_K_surface_ ind] = mbest(train_120_vq, rate_K_surface_no_mean, m);
-
-    for f=1:frames
-        rate_K_surface_(f,:) = post_filter(rate_K_surface_(f,:), 1.5, voicing(f));
+    Wo = model(f,1);
+    L = model(f,2);
+    Am = model(f,3:(L+2));
+    AmdB = 20*log10(Am);
+    Am_freqs_kHz = (1:L)*Wo*Fs/(2000*pi);
+    [rate_K_vec rate_K_sample_freqs_kHz] = resample_const_rate_f_mel(model(f,:), K, Fs);
+    if correct_rate_K_en
+      [tmp_ AmdB_] = resample_rate_L(model(f,:), rate_K_vec, rate_K_sample_freqs_kHz, Fs);
+      [rate_K_vec_corrected orig_error error nasty_error_log nasty_error_m_log] = correct_rate_K_vec(rate_K_vec, rate_K_sample_freqs_kHz, AmdB, AmdB_, K, Wo, L, Fs);
+      rate_K_surface(f,:) = rate_K_vec_corrected;
+    else
+      rate_K_surface(f,:) = rate_K_vec;
     end
-
-    for f=1:frames
-      rate_K_surface_(f,:) += mean_f(f);
-    end
-
-    rate_K_surface = rate_K_surface_;
   end
 
-  model_ = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz);
-
-  if plots
-    figure(3); clf; mesh(rate_K_surface_no_mean);
-    figure(4); clf; plot_dft_surface(rate_K_surface)
-  end
-
-  for f=1:frames
-    rate_K_surface(f,:) -= mean(rate_K_surface(f,:));
-  end
+  model_ = resample_rate_L(model, rate_K_surface, rate_K_sample_freqs_kHz, Fs);
+ 
 endfunction
 
 
+
 % mel spaced sampling, differential in time VQ.  Curiously, couldn't
 % get very good results out of this, I suspect a bug
 
@@ -922,6 +1246,24 @@ function model_ =  experiment_filter_dec_filter(model)
 endfunction
 
 
+% smoothing using masking functions
+
+function model_ = experiment_smoothed(model, bark_model=1)
+  [frames nc] = size(model);
+
+  model_ = model;
+
+  for f=1:frames
+    Wo = model(f,1);
+    L = model(f,2);
+    Am = model(f,3:(L+2));
+    AmdB = 20*log10(Am);
+    [AmdB_ Am_freqs_kHz] = mask_model(AmdB, Wo, L, bark_model);
+    model_(f,3:(L+2)) = 10 .^ ((AmdB_)/20);
+  end
+
+endfunction
+
 #{
   
   My original idea was to used a 3-4 "resonators" to construct a
@@ -931,9 +1273,11 @@ endfunction
   quantise.
 
   todo: get this working again
+#}
 
 function model_ = experiment_piecewise(model)
-  % encoder loop ------------------------------------------------------
+
+  [frames tmp] = size(model); max_amp = 160;
 
   fvec_log = []; amps_log = [];
 
@@ -951,31 +1295,11 @@ function model_ = experiment_piecewise(model)
     fvec_log = [fvec_log; fvec];
     amps_log = [amps_log; amps];
 
-    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB(1:L)/20);
+    model_(f,1) = Wo; model_(f,2) = L; model_(f,3:(L+2)) = 10 .^ (AmdB_(1:L)/20);
   end
 
-  for f=1:frames
-    if 0
-    if (f > 1) && (e(f) > (e(f-1)+3))
-        decimate = 2;
-      else
-        decimate = 4;
-    end
-    end
-    printf("%d ", decimate);   
-    
-    AmdB_ = decimate_frame_rate(model_, decimate, f, frames);
-    L = length(AmdB_);
-
-    Am_ = zeros(1,max_amp);
-    Am_(2:L) = 10 .^ (AmdB_(1:L-1)/20);  % C array doesnt use A[0]
-    fwrite(fam, Am_, "float32");
-    fwrite(fWo, Wo, "float32");
-  end
 endfunction
 
-#}
-
 
 % early test, devised to test rate K<->L changes along frequency axis
 
@@ -1057,3 +1381,4 @@ function plot_dft_surface(surface)
   Fs = 100; dF = Fs/frames;
   mesh(1:K, (1:frames/2)*dF, 20*log10(abs(dft_surface(1:frames/2,:,:))));
 endfunction
+
