@@ -50,10 +50,12 @@
 #include "phaseexp.h"
 #include "bpf.h"
 #include "bpfb.h"
+#include "newamp1.h"
 
-void synth_one_frame(codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
+void synth_one_frame(int n_samp, codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[], float Pn[], int prede, float *de_mem, float gain);
 void print_help(const struct option *long_options, int num_opts, char* argv[]);
 
+#define N_SAMP n_samp  /* quick fix for run time sample rate selection */
 
 /*---------------------------------------------------------------------------*\
 
@@ -63,30 +65,10 @@ void print_help(const struct option *long_options, int num_opts, char* argv[]);
 
 int main(int argc, char *argv[])
 {
-    FILE *fout = NULL;	/* output speech file                    */
-    FILE *fin;		/* input speech file                     */
-    short buf[N_SAMP];	/* input/output buffer                   */
-    float buf_float[N_SAMP];
-    float Sn[M_PITCH];	/* float input speech samples            */
-    float Sn_pre[M_PITCH];	/* pre-emphasised input speech samples   */
-    COMP  Sw[FFT_ENC];	/* DFT of Sn[]                           */
-    codec2_fft_cfg  fft_fwd_cfg;
-    codec2_fftr_cfg  fftr_fwd_cfg;
-    codec2_fftr_cfg  fftr_inv_cfg;
-    float w[M_PITCH];	        /* time domain hamming window            */
-    COMP  W[FFT_ENC];	/* DFT of w[]                            */
-    MODEL model;
-    float Pn[2*N_SAMP];	/* trapezoidal synthesis window          */
-    float Sn_[2*N_SAMP];	/* synthesised speech */
-    int   i,m;		/* loop variable                         */
-    int   frames;
-    float prev_Wo, prev__Wo, prev_uq_Wo;
-    float pitch;
-    char  out_file[MAX_STR];
-    char  ampexp_arg[MAX_STR];
-    char  phaseexp_arg[MAX_STR];
-    float snr;
-    float sum_snr;
+
+    int Fs = 8000;
+    int set_fs;
+
     int orderi;
     int lpc_model = 0, order = LPC_ORD;
     int lsp = 0, lspd = 0, lspvq = 0;
@@ -96,61 +78,42 @@ int main(int argc, char *argv[])
     int lspanssi = 0,
     #endif
     int prede = 0;
-    float pre_mem = 0.0, de_mem = 0.0;
-    float ak[order];
-    // COMP  Sw_[FFT_ENC];
-    // COMP  Ew[FFT_ENC];
-
-    int phase0 = 0;
-    float ex_phase[MAX_AMP+1];
-
     int   postfilt;
-    float bg_est = 0.0;
-
     int   hand_voicing = 0, phaseexp = 0, ampexp = 0, hi = 0, simlpcpf = 0, lspmelread = 0;
     int   lpcpf = 0;
     FILE *fvoicing = 0;
     FILE *flspmel = 0;
-
-    MODEL prev_model;
     int dec;
     int decimate = 1;
-    float lsps[order];
-    float e, prev_e;
-    int   lsp_indexes[order];
-    float lsps_[order];
-    float Woe_[2];
-
-    float lsps_dec[4][LPC_ORD], e_dec[4], weight, weight_inc, ak_dec[4][LPC_ORD];
-    MODEL model_dec[4], prev_model_dec;
-    float prev_lsps_dec[order], prev_e_dec;
-
-    void *nlp_states;
-    float hpf_states[2];
+    int   amread, Woread;
+    int   awread;
+    int   hmread;
+    int   phase0 = 0;
     int   scalar_quant_Wo_e = 0;
     int   scalar_quant_Wo_e_low = 0;
     int   vector_quant_Wo_e = 0;
     int   dump_pitch_e = 0;
+    float gain = 1.0;
+    int   bpf_en = 0;
+    int   bpfb_en = 0;
+    FILE *fam = NULL, *fWo = NULL;
+    FILE *faw = NULL;
+    FILE *fhm = NULL;
     FILE *fjvm = NULL;
     #ifdef DUMP
     int   dump;
     #endif
-    struct PEXP *pexp = NULL;
-    struct AEXP *aexp = NULL;
-    float gain = 1.0;
-    int   bpf_en = 0;
-    int   bpfb_en = 0;
-    float bpf_buf[BPF_N+N_SAMP];
-    float lspmelvq_mse = 0.0;
-    int   amread, Woread;
-    FILE *fam = NULL, *fWo = NULL;
-    int   awread;
-    FILE *faw = NULL;
-    int   hmread;
-    FILE *fhm = NULL;
+    char  ampexp_arg[MAX_STR];
+    char  phaseexp_arg[MAX_STR];
+    char  out_file[MAX_STR];
+    FILE *fout = NULL;	/* output speech file */
+    int   mel_resampling = 0;
+    int   K = 20;
 
     char* opt_string = "ho:";
     struct option long_options[] = {
+        { "Fs", required_argument, &set_fs, 1 },
+        { "mel", required_argument, &mel_resampling, 1 },
         { "lpc", required_argument, &lpc_model, 1 },
         { "lspjnd", no_argument, &lspjnd, 1 },
         { "lspmel", no_argument, &lspmel, 1 },
@@ -193,36 +156,6 @@ int main(int argc, char *argv[])
         { NULL, no_argument, NULL, 0 }
     };
     int num_opts=sizeof(long_options)/sizeof(struct option);
-    COMP Aw[FFT_ENC];
-    COMP H[MAX_AMP];
-
-    for(i=0; i<M_PITCH; i++) {
-	Sn[i] = 1.0;
-	Sn_pre[i] = 1.0;
-    }
-    for(i=0; i<2*N_SAMP; i++)
-	Sn_[i] = 0;
-
-    prev_uq_Wo = prev_Wo = prev__Wo = TWO_PI/P_MAX;
-
-    prev_model.Wo = TWO_PI/P_MIN;
-    prev_model.L = floor(PI/prev_model.Wo);
-    for(i=1; i<=prev_model.L; i++) {
-	prev_model.A[i] = 0.0;
-	prev_model.phi[i] = 0.0;
-    }
-    for(i=1; i<=MAX_AMP; i++) {
-	//ex_phase[i] = (PI/3)*(float)rand()/RAND_MAX;
-	ex_phase[i] = 0.0;
-    }
-    e = prev_e = 1;
-    hpf_states[0] = hpf_states[1] = 0.0;
-
-    nlp_states = nlp_create(M_PITCH);
-
-    if (argc < 2) {
-        print_help(long_options, num_opts, argv);
-    }
 
     /*----------------------------------------------------------------*\
 
@@ -230,6 +163,10 @@ int main(int argc, char *argv[])
 
     \*----------------------------------------------------------------*/
 
+    if (argc < 2) {
+        print_help(long_options, num_opts, argv);
+    }
+
     while(1) {
         int option_index = 0;
         int opt = getopt_long(argc, argv, opt_string,
@@ -238,7 +175,15 @@ int main(int argc, char *argv[])
             break;
         switch (opt) {
          case 0:
-            if(strcmp(long_options[option_index].name, "lpc") == 0) {
+            if(strcmp(long_options[option_index].name, "Fs") == 0) {
+                Fs= atoi(optarg);
+                if((Fs != 8000) && (Fs != 16000)) {
+                    fprintf(stderr, "Error Fs must be 8000 or 16000\n");
+                    exit(1);
+                }
+            } else if(strcmp(long_options[option_index].name, "mel") == 0) {
+                K = atoi(optarg);
+            } else if(strcmp(long_options[option_index].name, "lpc") == 0) {
                 orderi = atoi(optarg);
                 if((orderi < 4) || (orderi > order)) {
                     fprintf(stderr, "Error in LPC order (4 to %d): %s\n", order, optarg);
@@ -389,6 +334,7 @@ int main(int argc, char *argv[])
 
     /* Input file */
 
+    FILE *fin;		/* input speech file                     */
     if (strcmp(argv[optind], "-")  == 0) fin = stdin;
     else if ((fin = fopen(argv[optind],"rb")) == NULL) {
 	fprintf(stderr, "Error opening input speech file: %s: %s.\n",
@@ -396,6 +342,88 @@ int main(int argc, char *argv[])
 	exit(1);
     }
 
+    C2CONST c2const = c2const_create(Fs);
+    int   n_samp = c2const.n_samp;
+    int   m_pitch = c2const.m_pitch;
+
+    short buf[N_SAMP];	/* input/output buffer                   */
+    float buf_float[N_SAMP];
+    float Sn[m_pitch];	/* float input speech samples            */
+    float Sn_pre[m_pitch];	/* pre-emphasised input speech samples   */
+    COMP  Sw[FFT_ENC];	/* DFT of Sn[]                           */
+    codec2_fft_cfg  fft_fwd_cfg;
+    codec2_fftr_cfg  fftr_fwd_cfg;
+    codec2_fftr_cfg  fftr_inv_cfg;
+    float w[m_pitch];	        /* time domain hamming window            */
+    COMP  W[FFT_ENC];	/* DFT of w[]                            */
+    MODEL model;
+    float Pn[2*N_SAMP];	/* trapezoidal synthesis window          */
+    float Sn_[2*N_SAMP];	/* synthesised speech */
+    int   i,m;		/* loop variable                         */
+    int   frames;
+    float prev_f0;
+    float pitch;
+    float snr;
+    float sum_snr;
+
+    float pre_mem = 0.0, de_mem = 0.0;
+    float ak[order];
+    // COMP  Sw_[FFT_ENC];
+    // COMP  Ew[FFT_ENC];
+
+    float ex_phase[MAX_AMP+1];
+
+    float bg_est = 0.0;
+
+
+    MODEL prev_model;
+    float lsps[order];
+    float e, prev_e;
+    int   lsp_indexes[order];
+    float lsps_[order];
+    float Woe_[2];
+
+    float lsps_dec[4][LPC_ORD], e_dec[4], weight, weight_inc, ak_dec[4][LPC_ORD];
+    MODEL model_dec[4], prev_model_dec;
+    float prev_lsps_dec[order], prev_e_dec;
+
+    void *nlp_states;
+    float hpf_states[2];
+    #if 0
+    struct PEXP *pexp = NULL;
+    struct AEXP *aexp = NULL;
+    #endif
+    float bpf_buf[BPF_N+N_SAMP];
+    float lspmelvq_mse = 0.0;
+
+    COMP Aw[FFT_ENC];
+    COMP H[MAX_AMP];
+
+ 
+    for(i=0; i<m_pitch; i++) {
+	Sn[i] = 1.0;
+	Sn_pre[i] = 1.0;
+    }
+    for(i=0; i<2*N_SAMP; i++)
+	Sn_[i] = 0;
+
+    prev_f0 = 1/P_MAX_S;
+
+    prev_model.Wo = c2const.Wo_max;
+    prev_model.L = floor(PI/prev_model.Wo);
+    for(i=1; i<=prev_model.L; i++) {
+	prev_model.A[i] = 0.0;
+	prev_model.phi[i] = 0.0;
+    }
+    for(i=1; i<=MAX_AMP; i++) {
+	//ex_phase[i] = (PI/3)*(float)rand()/RAND_MAX;
+	ex_phase[i] = 0.0;
+    }
+    e = prev_e = 1;
+    hpf_states[0] = hpf_states[1] = 0.0;
+
+    nlp_states = nlp_create(&c2const);
+
     ex_phase[0] = 0;
     Woe_[0] = Woe_[1] = 1.0;
 
@@ -407,16 +435,23 @@ int main(int argc, char *argv[])
 
     /* Initialise ------------------------------------------------------------*/
 
-    fft_fwd_cfg = codec2_fft_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
+    fft_fwd_cfg = codec2_fft_alloc(FFT_ENC, 0, NULL, NULL);   /* fwd FFT,used in several places   */
     fftr_fwd_cfg = codec2_fftr_alloc(FFT_ENC, 0, NULL, NULL); /* fwd FFT,used in several places   */
     fftr_inv_cfg = codec2_fftr_alloc(FFT_DEC, 1, NULL, NULL); /* inverse FFT, used just for synth */
-    make_analysis_window(fft_fwd_cfg, w, W);
-    make_synthesis_window(Pn);
+    codec2_fft_cfg phase_fft_fwd_cfg = codec2_fft_alloc(NEWAMP1_PHASE_NFFT, 0, NULL, NULL);
+    codec2_fft_cfg phase_fft_inv_cfg = codec2_fft_alloc(NEWAMP1_PHASE_NFFT, 1, NULL, NULL);
+
+    make_analysis_window(&c2const, fft_fwd_cfg, w, W);
+    make_synthesis_window(&c2const, Pn);
     quantise_init();
+
+    /* disabled for now while we convert to runtime n_samp */
+    #if 0
     if (phaseexp)
 	pexp = phase_experiment_create();
     if (ampexp)
 	aexp = amp_experiment_create();
+    #endif
 
     if (bpfb_en)
         bpf_en = 1;
@@ -431,10 +466,21 @@ int main(int argc, char *argv[])
     prev_e_dec = 1;
     for(m=1; m<=MAX_AMP; m++)
 	prev_model_dec.A[m] = 0.0;
-    prev_model_dec.Wo = TWO_PI/P_MAX;
+    prev_model_dec.Wo = c2const.Wo_min;
     prev_model_dec.L = PI/prev_model_dec.Wo;
     prev_model_dec.voiced = 0;
 
+    /* mel resampling experiments */
+
+    float rate_K_sample_freqs_kHz[K];
+    if (mel_resampling) {
+        float mel_start = ftomel(100); 
+        float mel_end = ftomel(0.95*Fs/2);
+        mel_sample_freqs_kHz(rate_K_sample_freqs_kHz, K, mel_start, mel_end);
+        //for(i=0; i<K; i++)
+        //    fprintf(stderr, "%d %f\n", i, rate_K_sample_freqs_kHz[i]);
+    }
+
     /*----------------------------------------------------------------* \
 
                             Main Loop
@@ -476,11 +522,11 @@ int main(int argc, char *argv[])
 
         /* shift buffer of input samples, and insert new samples */
 
-	for(i=0; i<M_PITCH-N_SAMP; i++) {
+	for(i=0; i<m_pitch-N_SAMP; i++) {
 	    Sn[i] = Sn[i+N_SAMP];
 	}
 	for(i=0; i<N_SAMP; i++) {
-	    Sn[i+M_PITCH-N_SAMP] = buf_float[i];
+	    Sn[i+m_pitch-N_SAMP] = buf_float[i];
         }
 
 	/*------------------------------------------------------------*\
@@ -489,17 +535,18 @@ int main(int argc, char *argv[])
 
 	\*------------------------------------------------------------*/
 
-	nlp(nlp_states,Sn,N_SAMP,P_MIN,P_MAX,&pitch,Sw,W,&prev_uq_Wo);
+        nlp(nlp_states, Sn, N_SAMP, &pitch, Sw, W, &prev_f0);
 	model.Wo = TWO_PI/pitch;
 
-	dft_speech(fft_fwd_cfg, Sw, Sn, w);
-	two_stage_pitch_refinement(&model, Sw);
+        dft_speech(&c2const, fft_fwd_cfg, Sw, Sn, w);
+	two_stage_pitch_refinement(&c2const, &model, Sw);
 	estimate_amplitudes(&model, Sw, W, 1);
 
         #ifdef DUMP
-	dump_Sn(Sn); dump_Sw(Sw); dump_model(&model);
+        dump_Sn(m_pitch, Sn); dump_Sw(Sw); dump_model(&model);
         #endif
 
+        #if 0
 	if (ampexp)
 	    amp_experiment(aexp, &model, ampexp_arg);
 
@@ -512,6 +559,7 @@ int main(int argc, char *argv[])
 	    dump_phase_(&model.phi[0], model.L);
             #endif
 	}
+        #endif
 
 	if (hi) {
 	    int m;
@@ -521,6 +569,18 @@ int main(int argc, char *argv[])
 		model.A[m] = 0.0;
 	}
 
+	/*------------------------------------------------------------*\
+
+	                     Mel scale resampling
+
+	\*------------------------------------------------------------*/
+
+        if (mel_resampling) {
+            float rate_K_vec[K];
+            resample_const_rate_f(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+            resample_rate_L(&c2const, &model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+        }
+
 	/*------------------------------------------------------------*\
 
                             Zero-phase modelling
@@ -528,27 +588,16 @@ int main(int argc, char *argv[])
 	\*------------------------------------------------------------*/
 
 	if (phase0) {
-	    float Wn[M_PITCH];		        /* windowed speech samples */
-	    float Rk[order+1];                  /* autocorrelation coeffs  */
-            COMP a[FFT_ENC];
-
             #ifdef DUMP
 	    dump_phase(&model.phi[0], model.L);
             #endif
 
-	    /* find aks here, these are overwritten if LPC modelling is enabled */
-
-            for(i=0; i<M_PITCH; i++)
-                Wn[i] = Sn[i]*w[i];
-	    autocorrelate(Wn,Rk,M_PITCH,order);
-	    levinson_durbin(Rk,ak,order);
-
 	    /* determine voicing */
 
 	    #if 0
-	    snr = est_voicing_mbe(&model, Sw, W, Sw_, Ew);
+            snr = est_voicing_mbe(&c2const, &model, Sw, W, Sw_, Ew);
             #else
-	    snr = est_voicing_mbe(&model, Sw, W);
+	    snr = est_voicing_mbe(&c2const, &model, Sw, W);
             #endif
 
 	    if (dump_pitch_e)
@@ -567,18 +616,6 @@ int main(int argc, char *argv[])
 	    for(i=0; i<=MAX_AMP; i++)
 	    	model.phi[i] = 0;
 
-            /* Determine DFT of A(exp(jw)), which is needed for phase0 model when
-               LPC is not used, e.g. indecimate=1 (10ms) frames with no LPC */
-
-            for(i=0; i<FFT_ENC; i++) {
-                a[i].real = 0.0;
-                a[i].imag = 0.0;
-            }
-
-            for(i=0; i<=order; i++)
-                a[i].real = ak[i];
-            codec2_fft(fft_fwd_cfg, a, Aw);
-
 	    if (hand_voicing) {
 		int ret = fscanf(fvoicing,"%d\n",&model.voiced);
                 assert(ret == 1);
@@ -593,7 +630,7 @@ int main(int argc, char *argv[])
 
 	if (lpc_model) {
 
-            e = speech_to_uq_lsps(lsps, ak, Sn, w, order);
+            e = speech_to_uq_lsps(lsps, ak, Sn, w, m_pitch, order);
             for(i=0; i<LPC_ORD; i++)
                 lsps_[i] = lsps[i];
 
@@ -754,14 +791,14 @@ int main(int argc, char *argv[])
 	    if (scalar_quant_Wo_e) {
 
 		e = decode_energy(encode_energy(e, E_BITS), E_BITS);
-                model.Wo = decode_Wo(encode_Wo(model.Wo, WO_BITS), WO_BITS);
+                model.Wo = decode_Wo(&c2const, encode_Wo(&c2const, model.Wo, WO_BITS), WO_BITS);
 		model.L  = PI/model.Wo; /* if we quantise Wo re-compute L */
 	    }
 
 	    if (scalar_quant_Wo_e_low) {
                 int ind;
 		e = decode_energy(ind = encode_energy(e, 3), 3);
-                model.Wo = decode_log_Wo(encode_log_Wo(model.Wo, 5), 5);
+                model.Wo = decode_log_Wo(&c2const, encode_log_Wo(&c2const, model.Wo, 5), 5);
 		model.L  = PI/model.Wo; /* if we quantise Wo re-compute L */
 	    }
 
@@ -769,7 +806,7 @@ int main(int argc, char *argv[])
 
 		/* JVM's experimental joint Wo & LPC energy quantiser */
 
-		quantise_WoE(&model, &e, Woe_);
+		quantise_WoE(&c2const, &model, &e, Woe_);
 	    }
 
 	}
@@ -817,7 +854,7 @@ int main(int argc, char *argv[])
             for(i=0, weight=weight_inc; i<decimate-1; i++, weight += weight_inc) {
                 //model_dec[i].voiced = model_dec[decimate-1].voiced;
                 interpolate_lsp_ver2(&lsps_dec[i][0], prev_lsps_dec, &lsps_dec[decimate-1][0], weight, order);
-                interp_Wo2(&model_dec[i], &prev_model_dec, &model_dec[decimate-1], weight);
+                interp_Wo2(&model_dec[i], &prev_model_dec, &model_dec[decimate-1], weight, c2const.Wo_min);
                 e_dec[i] = interp_energy2(prev_e_dec, e_dec[decimate-1],weight);
             }
 
@@ -851,13 +888,15 @@ int main(int argc, char *argv[])
                         int ret = fread(H, sizeof(COMP), MAX_AMP, fhm);
                         assert(ret == MAX_AMP);
                     } else {
-                        sample_phase(&model_dec[i], H, Aw);
+                        determine_phase(&c2const, H, &model_dec[i], NEWAMP1_PHASE_NFFT, phase_fft_fwd_cfg, phase_fft_inv_cfg);
                     }
-                    phase_synth_zero_order(&model_dec[i], ex_phase, H);
+                    phase_synth_zero_order(n_samp, &model_dec[i], ex_phase, H);
                 }
+
+                
                 if (postfilt)
                     postfilter(&model_dec[i], &bg_est);
-                synth_one_frame(fftr_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
+                synth_one_frame(n_samp, fftr_inv_cfg, buf, &model_dec[i], Sn_, Pn, prede, &de_mem, gain);
                 if (fout != NULL) fwrite(buf,sizeof(short),N_SAMP,fout);
             }
 
@@ -888,10 +927,12 @@ int main(int argc, char *argv[])
             fprintf(stderr, "lspmelvq std = %3.1f Hz\n", sqrt(lspmelvq_mse/frames));
     }
 
+    #if 0
     if (phaseexp)
 	phase_experiment_destroy(pexp);
     if (ampexp)
 	amp_experiment_destroy(aexp);
+    #endif
     #ifdef DUMP
     if (dump)
 	dump_off();
@@ -905,16 +946,16 @@ int main(int argc, char *argv[])
     return 0;
 }
 
-void synth_one_frame(codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[],
+void synth_one_frame(int n_samp, codec2_fftr_cfg fftr_inv_cfg, short buf[], MODEL *model, float Sn_[],
                      float Pn[], int prede, float *de_mem, float gain)
 {
     int     i;
 
-    synthesise(fftr_inv_cfg, Sn_, model, Pn, 1);
+    synthesise(n_samp, fftr_inv_cfg, Sn_, model, Pn, 1);
     if (prede)
-        de_emp(Sn_, Sn_, de_mem, N_SAMP);
+        de_emp(Sn_, Sn_, de_mem, n_samp);
 
-    for(i=0; i<N_SAMP; i++) {
+    for(i=0; i<n_samp; i++) {
 	Sn_[i] *= gain;
 	if (Sn_[i] > 32767.0)
 	    buf[i] = 32767;
