@@ -34,22 +34,26 @@
 
 struct CODEC2 {
     int           mode;
-    codec2_fft_cfg  fft_fwd_cfg;             /* forward FFT config                        */
-    codec2_fftr_cfg fftr_fwd_cfg;            /* forward real FFT config                   */
-    float         w[M_PITCH];	                   /* time domain hamming window                */
+    C2CONST       c2const;
+    int           Fs;
+    int           n_samp;
+    int           m_pitch;
+    codec2_fft_cfg  fft_fwd_cfg;           /* forward FFT config                        */
+    codec2_fftr_cfg fftr_fwd_cfg;          /* forward real FFT config                   */
+    float        *w;	                   /* [m_pitch] time domain hamming window      */
     COMP          W[FFT_ENC];	           /* DFT of w[]                                */
-    float         Pn[2*N_SAMP];	           /* trapezoidal synthesis window              */
+    float        *Pn;	                   /* [2*n_samp] trapezoidal synthesis window   */
     float        *bpf_buf;                 /* buffer for band pass filter               */
-    float         Sn[M_PITCH];                   /* input speech                              */
+    float        *Sn;                      /* [m_pitch] input speech                    */
     float         hpf_states[2];           /* high pass filter states                   */
     void         *nlp;                     /* pitch predictor states                    */
     int           gray;                    /* non-zero for gray encoding                */
 
-    codec2_fftr_cfg  fftr_inv_cfg;             /* inverse FFT config                        */
-    float         Sn_[2*N_SAMP];	           /* synthesised output speech                 */
+    codec2_fftr_cfg  fftr_inv_cfg;         /* inverse FFT config                        */
+    float        *Sn_;	                   /* [2*n_samp] synthesised output speech      */
     float         ex_phase;                /* excitation model phase track              */
     float         bg_est;                  /* background noise estimate for post filter */
-    float         prev_Wo_enc;             /* previous frame's pitch estimate           */
+    float         prev_f0_enc;             /* previous frame's f0    estimate           */
     MODEL         prev_model_dec;          /* previous frame's model parameters         */
     float         prev_lsps_dec[LPC_ORD];  /* previous frame's LSPs                     */
     float         prev_e_dec;              /* previous frame's LPC energy               */
