@@ -150,6 +150,10 @@ struct COHPSK *cohpsk_create(void)
             coh->rx_filter_memory[c][i].real = 0.0;
             coh->rx_filter_memory[c][i].imag = 0.0;
         }
+
+        /* optional per-carrier amplitude weighting for testing */
+
+        coh->carrier_ampl[c] = 1.0;
     }
     fdmdv->fbb_rect.real     = cosf(2.0*PI*FDMDV_FCENTRE/COHPSK_FS);
     fdmdv->fbb_rect.imag     = sinf(2.0*PI*FDMDV_FCENTRE/COHPSK_FS);
@@ -181,7 +185,7 @@ struct COHPSK *cohpsk_create(void)
 
     /* test frames */
 
-    coh->ptest_bits_coh_tx = coh->ptest_bits_coh_rx = (int*)test_bits_coh;
+    coh->ptest_bits_coh_tx = coh->ptest_bits_coh_rx[0] = coh->ptest_bits_coh_rx[1] = (int*)test_bits_coh;
     coh->ptest_bits_coh_end = (int*)test_bits_coh + sizeof(test_bits_coh)/sizeof(int);
 
     return coh;
@@ -218,12 +222,22 @@ void cohpsk_destroy(struct COHPSK *coh)
 
 void bits_to_qpsk_symbols(COMP tx_symb[][COHPSK_NC*ND], int tx_bits[], int nbits)
 {
-    int   i, r, c, p_r, data_r, d;
+    int   i, r, c, p_r, data_r, d, diversity;
     short bits;
 
-    /* check number of bits supplied matchs number of QPSK symbols in the frame */
+    /* check allowed number of bits supplied matches number of QPSK
+       symbols in the frame */
 
-    assert((NSYMROW*COHPSK_NC)*2 == nbits);
+    assert( (NSYMROW*COHPSK_NC*2 == nbits) || (NSYMROW*COHPSK_NC*2*ND == nbits));
+    
+    /* if we input twice as many bits we don't do diversity */
+
+    if (NSYMROW*COHPSK_NC*2 == nbits) {    
+        diversity = 1; /* diversity mode                         */
+    }
+    else {
+        diversity = 2; /* twice as many bits, non diversity mode */
+    }
 
     /*
       Insert two rows of Nc pilots at beginning of data frame.
@@ -240,44 +254,32 @@ void bits_to_qpsk_symbols(COMP tx_symb[][COHPSK_NC*ND], int tx_bits[], int nbits
 
     r = 0;
     for(p_r=0; p_r<2; p_r++) {
-        for(c=0; c<COHPSK_NC; c++) {
-            tx_symb[r][c].real = pilots_coh[p_r][c]/sqrtf(ND);
+        for(c=0; c<COHPSK_NC*ND; c++) {
+            tx_symb[r][c].real = pilots_coh[p_r][c % COHPSK_NC]/sqrtf(ND);
             tx_symb[r][c].imag = 0.0;
         }
         r++;
     }
     for(data_r=0; data_r<NSYMROW; data_r++, r++) {
-
-        for(c=0; c<COHPSK_NC; c++) {
+        for(c=0; c<COHPSK_NC*diversity; c++) {
             i = c*NSYMROW + data_r;
             bits = (tx_bits[2*i]&0x1)<<1 | (tx_bits[2*i+1]&0x1);
             tx_symb[r][c] = fcmult(1.0/sqrtf(ND),qpsk_mod[bits]);
-
-            /* test code to see what happens when we attenuate one
-               carrier, this may happen in practice with tx SSB filter
-               ripple.
-
-               if (c==5) {
-                 COMP attn = {0.5,0.0};
-                 tx_symb[r][c] = cmult(tx_symb[r][c], attn);
-               }
-            */
         }
     }
 
     assert(p_r == NPILOTSFRAME);
     assert(r == NSYMROWPILOT);
 
-    /* copy to other carriers (diversity) */
+    /* if in diversity mode, copy symbols to upper carriers */
 
-    for(d=1; d<ND; d++) {
+    for(d=1; d<1+ND-diversity; d++) {
         for(r=0; r<NSYMROWPILOT; r++) {
             for(c=0; c<COHPSK_NC; c++) {
                 tx_symb[r][c+COHPSK_NC*d] = tx_symb[r][c];
             }
         }
     }
-
 }
 
 
@@ -362,6 +364,18 @@ void qpsk_symbols_to_bits(struct COHPSK *coh, float rx_bits[], COMP ct_symb_buf[
             i = c*NSYMROW + r;
             rx_bits[2*i+1] = rot.real;
             rx_bits[2*i]   = rot.imag;
+
+            /* demodulate bits from upper and lower carriers separately for test purposes */
+
+            assert(ND == 2);
+
+            i = c*NSYMROW + r;
+            rot = cmult(coh->rx_symb[r][c], pi_on_4);
+            coh->rx_bits_lower[2*i+1] = rot.real;
+            coh->rx_bits_lower[2*i]   = rot.imag;
+            rot = cmult(coh->rx_symb[r][c + COHPSK_NC], pi_on_4);
+            coh->rx_bits_upper[2*i+1] = rot.real;
+            coh->rx_bits_upper[2*i]   = rot.imag;
         }
     }
 
@@ -648,8 +662,12 @@ int sync_state_machine(struct COHPSK *coh, int sync, int next_sync)
   AUTHOR......: David Rowe
   DATE CREATED: 5/4/2015
 
-  COHPSK modulator, take a frame of COHPSK_BITS_PER_FRAME bits and
-  generates a frame of COHPSK_NOM_SAMPLES_PER_FRAME modulated symbols.
+  COHPSK modulator, take a frame of COHPSK_BITS_PER_FRAME or
+  2*COHPSK_BITS_PER_FRAME bits and generates a frame of
+  COHPSK_NOM_SAMPLES_PER_FRAME modulated symbols.
+
+  if nbits == COHPSK_BITS_PER_FRAME, diveristy mode is used, if nbits
+  == 2*COHPSK_BITS_PER_FRAME diversity mode is not used.
 
   The output signal is complex to support single sided frequency
   shifting, for example when testing frequency offsets in channel
@@ -657,18 +675,18 @@ int sync_state_machine(struct COHPSK *coh, int sync, int next_sync)
 
 \*---------------------------------------------------------------------------*/
 
-void cohpsk_mod(struct COHPSK *coh, COMP tx_fdm[], int tx_bits[])
+void cohpsk_mod(struct COHPSK *coh, COMP tx_fdm[], int tx_bits[], int nbits)
 {
     struct FDMDV *fdmdv = coh->fdmdv;
     COMP  tx_symb[NSYMROWPILOT][COHPSK_NC*ND];
     COMP  tx_onesym[COHPSK_NC*ND];
     int  r,c;
 
-    bits_to_qpsk_symbols(tx_symb, tx_bits, COHPSK_BITS_PER_FRAME);
+    bits_to_qpsk_symbols(tx_symb, tx_bits, nbits);
 
     for(r=0; r<NSYMROWPILOT; r++) {
         for(c=0; c<COHPSK_NC*ND; c++)
-            tx_onesym[c] = tx_symb[r][c];
+            tx_onesym[c] = fcmult(coh->carrier_ampl[c], tx_symb[r][c]);
         tx_filter_and_upconvert_coh(&tx_fdm[r*COHPSK_M], COHPSK_NC*ND , tx_onesym, fdmdv->tx_filter_memory,
                                     fdmdv->phase_tx, fdmdv->freq, &fdmdv->fbb_phase_tx, fdmdv->fbb_rect);
     }
@@ -1195,23 +1213,29 @@ void cohpsk_get_test_bits(struct COHPSK *coh, int rx_bits[])
   Accepts bits from demod and attempts to sync with the known
   test_bits sequence.  When synced measures bit errors.
 
+  Has states to track two separate received test sequences based on
+  channel 0 or 1.
+
 \*---------------------------------------------------------------------------*/
 
 void cohpsk_put_test_bits(struct COHPSK *coh, int *state, short error_pattern[],
-			 int *bit_errors, char rx_bits_char[])
+                          int *bit_errors, char rx_bits_char[], int channel)
 {
     int i, next_state, anerror;
     int rx_bits[COHPSK_BITS_PER_FRAME];
 
+    assert((channel == 0) || (channel == 1));
+    int *ptest_bits_coh_rx = coh->ptest_bits_coh_rx[channel];
+
     for(i=0; i<COHPSK_BITS_PER_FRAME; i++) {
         rx_bits[i] = rx_bits_char[i];
     }
 
     *bit_errors = 0;
     for(i=0; i<COHPSK_BITS_PER_FRAME; i++) {
-        anerror = (rx_bits[i] & 0x1) ^ coh->ptest_bits_coh_rx[i];
+        anerror = (rx_bits[i] & 0x1) ^ ptest_bits_coh_rx[i];
         if ((anerror < 0) || (anerror > 1)) {
-            fprintf(stderr, "i: %d rx_bits: %d ptest_bits_coh_rx: %d\n", i, rx_bits[i], coh->ptest_bits_coh_rx[i]);
+            fprintf(stderr, "i: %d rx_bits: %d ptest_bits_coh_rx: %d\n", i, rx_bits[i], ptest_bits_coh_rx[i]);
         }
         *bit_errors += anerror;
         error_pattern[i] = anerror;
@@ -1224,9 +1248,9 @@ void cohpsk_put_test_bits(struct COHPSK *coh, int *state, short error_pattern[],
     if (*state == 0) {
         if (*bit_errors < 4) {
             next_state = 1;
-            coh->ptest_bits_coh_rx += COHPSK_BITS_PER_FRAME;
-            if (coh->ptest_bits_coh_rx >= coh->ptest_bits_coh_end) {
-                coh->ptest_bits_coh_rx = (int*)test_bits_coh;
+            ptest_bits_coh_rx += COHPSK_BITS_PER_FRAME;
+            if (ptest_bits_coh_rx >= coh->ptest_bits_coh_end) {
+                ptest_bits_coh_rx = (int*)test_bits_coh;
             }
         }
     }
@@ -1245,19 +1269,35 @@ void cohpsk_put_test_bits(struct COHPSK *coh, int *state, short error_pattern[],
     }
 
     if (*state > 0) {
-        coh->ptest_bits_coh_rx += COHPSK_BITS_PER_FRAME;
-        if (coh->ptest_bits_coh_rx >= coh->ptest_bits_coh_end) {
-            coh->ptest_bits_coh_rx = (int*)test_bits_coh;
+        ptest_bits_coh_rx += COHPSK_BITS_PER_FRAME;
+        if (ptest_bits_coh_rx >= coh->ptest_bits_coh_end) {
+            ptest_bits_coh_rx = (int*)test_bits_coh;
         }
     }
 
     //fprintf(stderr, "state: %d next_state: %d bit_errors: %d\n", *state, next_state, *bit_errors);
 
     *state = next_state;
+    coh->ptest_bits_coh_rx[channel] = ptest_bits_coh_rx;
 }
 
+
 int cohpsk_error_pattern_size(void) {
     return COHPSK_BITS_PER_FRAME;
 }
 
 
+float *cohpsk_get_rx_bits_lower(struct COHPSK *coh) {
+    return coh->rx_bits_lower;
+}
+
+float *cohpsk_get_rx_bits_upper(struct COHPSK *coh) {
+    return coh->rx_bits_upper;
+}
+
+void cohpsk_set_carrier_ampl(struct COHPSK *coh, int c, float ampl) {
+    assert(c < COHPSK_NC*ND);
+    coh->carrier_ampl[c] = ampl;
+    fprintf(stderr, "cohpsk_set_carrier_ampl: %d %f\n", c, ampl);
+}
+
