@@ -36,28 +36,26 @@
 
 /* General defines */
 
-#define N_SAMP          80		/* number of samples per frame          */
-#define MAX_AMP    80		/* maximum number of harmonics          */
+#define N_S        0.010        /* buffer size in s                     */
+#define TW_S       0.005        /* trapezoidal synth window overlap     */
+#define MAX_AMP    160		/* maximum number of harmonics          */
 #ifndef PI
 #define PI         3.141592654	/* mathematical constant                */
 #endif
 #define TWO_PI     6.283185307	/* mathematical constant                */
-#define FS         8000		/* sample rate in Hz                    */
-#define MAX_STR    256          /* maximum string size                  */
+#define MAX_STR    2048         /* maximum string size                  */
 
-#define NW         279          /* analysis window size                 */
 #define FFT_ENC    512		/* size of FFT used for encoder         */
 #define FFT_DEC    512	    	/* size of FFT used in decoder          */
-#define TW         40		/* Trapezoidal synthesis window overlap */
 #define V_THRESH   6.0          /* voicing threshold in dB              */
 #define LPC_ORD    10		/* LPC order                            */
 #define LPC_ORD_LOW 6		/* LPC order for lower rates            */
 
 /* Pitch estimation defines */
 
-#define M_PITCH        320		/* pitch analysis frame size            */
-#define P_MIN    20		/* minimum pitch                        */
-#define P_MAX    160		/* maximum pitch                        */
+#define M_PITCH_S  0.0400       /* pitch analysis window in s           */
+#define P_MIN_S    0.0025	/* minimum pitch period in s            */
+#define P_MAX_S    0.0200	/* maximum pitch period in s            */
 
 /*---------------------------------------------------------------------------*\
 
@@ -65,14 +63,29 @@
 
 \*---------------------------------------------------------------------------*/
 
+/* Structure to hold constants calculated at run time based on sample rate */
+
+typedef struct {
+    int   Fs;            /* sample rate of this instance             */
+    int   n_samp;        /* number of samples per 10ms frame at Fs   */
+    int   max_amp;       /* maximum number of harmonics              */
+    int   m_pitch;       /* pitch estimation window size in samples  */
+    int   p_min;         /* minimum pitch period in samples          */
+    int   p_max;         /* maximum pitch period in samples          */
+    float Wo_min;        
+    float Wo_max;  
+    int   nw;            /* analysis window size in samples          */      
+    int   tw;            /* trapezoidal synthesis window overlap     */
+} C2CONST;
+
 /* Structure to hold model parameters for one frame */
 
 typedef struct {
-  float Wo;		/* fundamental frequency estimate in radians  */
-  int   L;		/* number of harmonics                        */
-  float A[MAX_AMP+1];	/* amplitiude of each harmonic                */
-  float phi[MAX_AMP+1];	/* phase of each harmonic                     */
-  int   voiced;	        /* non-zero if this frame is voiced           */
+    float Wo;		  /* fundamental frequency estimate in radians  */
+    int   L;		  /* number of harmonics                        */
+    float A[MAX_AMP+1];	  /* amplitiude of each harmonic                */
+    float phi[MAX_AMP+1]; /* phase of each harmonic                     */
+    int   voiced;	  /* non-zero if this frame is voiced           */
 } MODEL;
 
 /* describes each codebook  */
