@@ -37,6 +37,13 @@
 #include <string.h>
 #include <math.h>
 
+#ifdef ARM_MATH_CM4
+  #include "stm32f4xx.h"
+  #include "core_cm4.h"
+  #include "arm_math.h"
+#endif
+
+
 #include "fdmdv_internal.h"
 #include "codec2_fdmdv.h"
 #include "comp_prim.h"
@@ -44,17 +51,27 @@
 #include "rxdec_coeff.h"
 #include "test_bits.h"
 #include "pilot_coeff.h"
-#include "kiss_fft.h"
+#include "codec2_fft.h"
 #include "hanning.h"
 #include "os.h"
 #include "machdep.h"
 
 static int sync_uw[] = {1,-1,1,-1,1,-1};
-
 #ifdef __EMBEDDED__
 #define printf gdb_stdio_printf
 #endif
 
+#ifndef ARM_MATH_CM4
+  #define SINF(a) sinf(a)
+  #define COSF(a) cosf(a)
+#else
+  #define SINF(a) arm_sin_f32(a)
+  #define COSF(a) arm_cos_f32(a)
+#endif
+
+static const COMP  pi_on_4 = { .70710678118654752439, .70710678118654752439 }; // COSF(PI/4) , SINF(PI/4)
+
+
 /*--------------------------------------------------------------------------* \
 
   FUNCTION....: fdmdv_create
@@ -74,8 +91,8 @@ struct FDMDV * fdmdv_create(int Nc)
 
     assert(NC == FDMDV_NC_MAX);  /* check public and private #defines match */
     assert(Nc <= NC);
-    assert(FDMDV_NOM_SAMPLES_PER_FRAME == M);
-    assert(FDMDV_MAX_SAMPLES_PER_FRAME == (M+M/P));
+    assert(FDMDV_NOM_SAMPLES_PER_FRAME == M_FAC);
+    assert(FDMDV_MAX_SAMPLES_PER_FRAME == (M_FAC+M_FAC/P));
 
     f = (struct FDMDV*)malloc(sizeof(struct FDMDV));
     if (f == NULL)
@@ -110,8 +127,8 @@ struct FDMDV * fdmdv_create(int Nc)
            This helped PAPR for a few dB.  We don't need to adjust rx
            phase as DQPSK takes care of that. */
 
-	f->phase_tx[c].real = cosf(2.0*PI*c/(Nc+1));
- 	f->phase_tx[c].imag = sinf(2.0*PI*c/(Nc+1));
+	f->phase_tx[c].real = COSF(2.0*PI*c/(Nc+1));
+ 	f->phase_tx[c].imag = SINF(2.0*PI*c/(Nc+1));
 
 	f->phase_rx[c].real = 1.0;
  	f->phase_rx[c].imag = 0.0;
@@ -124,12 +141,12 @@ struct FDMDV * fdmdv_create(int Nc)
     f->prev_tx_symbols[Nc].real = 2.0;
 
     fdmdv_set_fsep(f, FSEP);
-    f->freq[Nc].real = cosf(2.0*PI*0.0/FS);
-    f->freq[Nc].imag = sinf(2.0*PI*0.0/FS);
+    f->freq[Nc].real = COSF(2.0*PI*0.0/FS);
+    f->freq[Nc].imag = SINF(2.0*PI*0.0/FS);
     f->freq_pol[Nc]  = 2.0*PI*0.0/FS;
 
-    f->fbb_rect.real     = cosf(2.0*PI*FDMDV_FCENTRE/FS);
-    f->fbb_rect.imag     = sinf(2.0*PI*FDMDV_FCENTRE/FS);
+    f->fbb_rect.real     = COSF(2.0*PI*FDMDV_FCENTRE/FS);
+    f->fbb_rect.imag     = SINF(2.0*PI*FDMDV_FCENTRE/FS);
     f->fbb_pol           = 2.0*PI*FDMDV_FCENTRE/FS;
     f->fbb_phase_tx.real = 1.0;
     f->fbb_phase_tx.imag = 0.0;
@@ -142,7 +159,7 @@ struct FDMDV * fdmdv_create(int Nc)
 
     /* freq Offset estimation states */
 
-    f->fft_pilot_cfg = kiss_fft_alloc (MPILOTFFT, 0, NULL, NULL);
+    f->fft_pilot_cfg = codec2_fft_alloc (MPILOTFFT, 0, NULL, NULL);
     assert(f->fft_pilot_cfg != NULL);
 
     for(i=0; i<NPILOTBASEBAND; i++) {
@@ -150,9 +167,9 @@ struct FDMDV * fdmdv_create(int Nc)
 	f->pilot_baseband1[i].imag = f->pilot_baseband2[i].imag = 0.0;
     }
     f->pilot_lut_index = 0;
-    f->prev_pilot_lut_index = 3*M;
+    f->prev_pilot_lut_index = 3*M_FAC;
 
-    for(i=0; i<NRXDEC-1+M; i++) {
+    for(i=0; i<NRXDEC-1+M_FAC; i++) {
         f->rxdec_lpf_mem[i].real = 0.0;
         f->rxdec_lpf_mem[i].imag = 0.0;
     }
@@ -166,7 +183,7 @@ struct FDMDV * fdmdv_create(int Nc)
     f->foff_phase_rect.real = 1.0;
     f->foff_phase_rect.imag = 0.0;
 
-    for(i=0; i<NFILTER+M; i++) {
+    for(i=0; i<NFILTER+M_FAC; i++) {
         f->rx_fdm_mem[i].real = 0.0;
         f->rx_fdm_mem[i].imag = 0.0;
     }
@@ -201,7 +218,7 @@ struct FDMDV * fdmdv_create(int Nc)
 void fdmdv_destroy(struct FDMDV *fdmdv)
 {
     assert(fdmdv != NULL);
-    KISS_FFT_FREE(fdmdv->fft_pilot_cfg);
+    codec2_fft_free(fdmdv->fft_pilot_cfg);
     free(fdmdv->rx_test_bits_mem);
     free(fdmdv);
 }
@@ -257,15 +274,15 @@ void fdmdv_set_fsep(struct FDMDV *f, float fsep) {
 
     for(c=0; c<f->Nc/2; c++) {
 	carrier_freq = (-f->Nc/2 + c)*f->fsep;
-	f->freq[c].real = cosf(2.0*PI*carrier_freq/FS);
- 	f->freq[c].imag = sinf(2.0*PI*carrier_freq/FS);
+	f->freq[c].real = COSF(2.0*PI*carrier_freq/FS);
+ 	f->freq[c].imag = SINF(2.0*PI*carrier_freq/FS);
  	f->freq_pol[c]  = 2.0*PI*carrier_freq/FS;
     }
 
     for(c=f->Nc/2; c<f->Nc; c++) {
 	carrier_freq = (-f->Nc/2 + c + 1)*f->fsep;
-	f->freq[c].real = cosf(2.0*PI*carrier_freq/FS);
- 	f->freq[c].imag = sinf(2.0*PI*carrier_freq/FS);
+	f->freq[c].real = COSF(2.0*PI*carrier_freq/FS);
+ 	f->freq[c].imag = SINF(2.0*PI*carrier_freq/FS);
  	f->freq_pol[c]  = 2.0*PI*carrier_freq/FS;
     }
 }
@@ -333,12 +350,12 @@ void bits_to_dqpsk_symbols(COMP tx_symbols[], int Nc, COMP prev_tx_symbols[], in
   AUTHOR......: David Rowe
   DATE CREATED: 17/4/2012
 
-  Given Nc*NB bits construct M samples (1 symbol) of Nc+1 filtered
+  Given Nc*NB bits construct M_FAC samples (1 symbol) of Nc+1 filtered
   symbols streams.
 
 \*---------------------------------------------------------------------------*/
 
-void tx_filter(COMP tx_baseband[NC+1][M], int Nc, COMP tx_symbols[], COMP tx_filter_memory[NC+1][NSYM])
+void tx_filter(COMP tx_baseband[NC+1][M_FAC], int Nc, COMP tx_symbols[], COMP tx_filter_memory[NC+1][NSYM])
 {
     int     c;
     int     i,j,k;
@@ -352,25 +369,25 @@ void tx_filter(COMP tx_baseband[NC+1][M], int Nc, COMP tx_symbols[], COMP tx_fil
 	tx_filter_memory[c][NSYM-1] = cmult(tx_symbols[c], gain);
 
     /*
-       tx filter each symbol, generate M filtered output samples for each symbol.
+       tx filter each symbol, generate M_FAC filtered output samples for each symbol.
        Efficient polyphase filter techniques used as tx_filter_memory is sparse
     */
 
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_FAC; i++) {
 	for(c=0; c<Nc+1; c++) {
 
 	    /* filter real sample of symbol for carrier c */
 
 	    acc = 0.0;
-	    for(j=0,k=M-i-1; j<NSYM; j++,k+=M)
-		acc += M * tx_filter_memory[c][j].real * gt_alpha5_root[k];
+	    for(j=0,k=M_FAC-i-1; j<NSYM; j++,k+=M_FAC)
+		acc += M_FAC * tx_filter_memory[c][j].real * gt_alpha5_root[k];
 	    tx_baseband[c][i].real = acc;
 
 	    /* filter imag sample of symbol for carrier c */
 
 	    acc = 0.0;
-	    for(j=0,k=M-i-1; j<NSYM; j++,k+=M)
-		acc += M * tx_filter_memory[c][j].imag * gt_alpha5_root[k];
+	    for(j=0,k=M_FAC-i-1; j<NSYM; j++,k+=M_FAC)
+		acc += M_FAC * tx_filter_memory[c][j].imag * gt_alpha5_root[k];
 	    tx_baseband[c][i].imag = acc;
 
 	}
@@ -395,7 +412,7 @@ void tx_filter(COMP tx_baseband[NC+1][M], int Nc, COMP tx_symbols[], COMP tx_fil
   AUTHOR......: David Rowe
   DATE CREATED: 13 August 2014
 
-  Given Nc symbols construct M samples (1 symbol) of Nc+1 filtered
+  Given Nc symbols construct M_FAC samples (1 symbol) of Nc+1 filtered
   and upconverted symbols.
 
 \*---------------------------------------------------------------------------*/
@@ -416,7 +433,7 @@ void tx_filter_and_upconvert(COMP tx_fdm[], int Nc, COMP tx_symbols[],
     gain.real = sqrtf(2.0)/2.0;
     gain.imag = 0.0;
 
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_FAC; i++) {
 	tx_fdm[i].real = 0.0;
 	tx_fdm[i].imag = 0.0;
     }
@@ -425,27 +442,27 @@ void tx_filter_and_upconvert(COMP tx_fdm[], int Nc, COMP tx_symbols[],
 	tx_filter_memory[c][NSYM-1] = cmult(tx_symbols[c], gain);
 
     /*
-       tx filter each symbol, generate M filtered output samples for
+       tx filter each symbol, generate M_FAC filtered output samples for
        each symbol, which we then freq shift and sum with other
        carriers.  Efficient polyphase filter techniques used as
        tx_filter_memory is sparse
     */
 
     for(c=0; c<Nc+1; c++) {
-        for(i=0; i<M; i++) {
+        for(i=0; i<M_FAC; i++) {
 
 	    /* filter real sample of symbol for carrier c */
 
 	    acc = 0.0;
-	    for(j=0,k=M-i-1; j<NSYM; j++,k+=M)
-		acc += M * tx_filter_memory[c][j].real * gt_alpha5_root[k];
+	    for(j=0,k=M_FAC-i-1; j<NSYM; j++,k+=M_FAC)
+		acc += M_FAC * tx_filter_memory[c][j].real * gt_alpha5_root[k];
 	    tx_baseband.real = acc;
 
 	    /* filter imag sample of symbol for carrier c */
 
 	    acc = 0.0;
-	    for(j=0,k=M-i-1; j<NSYM; j++,k+=M)
-		acc += M * tx_filter_memory[c][j].imag * gt_alpha5_root[k];
+	    for(j=0,k=M_FAC-i-1; j<NSYM; j++,k+=M_FAC)
+		acc += M_FAC * tx_filter_memory[c][j].imag * gt_alpha5_root[k];
 	    tx_baseband.imag = acc;
 
             /* freq shift and sum */
@@ -457,7 +474,7 @@ void tx_filter_and_upconvert(COMP tx_fdm[], int Nc, COMP tx_symbols[],
 
     /* shift whole thing up to carrier freq */
 
-    for (i=0; i<M; i++) {
+    for (i=0; i<M_FAC; i++) {
 	*fbb_phase = cmult(*fbb_phase, fbb_rect);
 	tx_fdm[i] = cmult(tx_fdm[i], *fbb_phase);
     }
@@ -469,7 +486,7 @@ void tx_filter_and_upconvert(COMP tx_fdm[], int Nc, COMP tx_symbols[],
       shifting for the purpose of testing easier
     */
 
-    for (i=0; i<M; i++)
+    for (i=0; i<M_FAC; i++)
 	tx_fdm[i] = cmult(two, tx_fdm[i]);
 
     /* normalise digital oscillators as the magnitude can drift over time */
@@ -509,27 +526,27 @@ void tx_filter_and_upconvert(COMP tx_fdm[], int Nc, COMP tx_symbols[],
 
 \*---------------------------------------------------------------------------*/
 
-void fdm_upconvert(COMP tx_fdm[], int Nc, COMP tx_baseband[NC+1][M], COMP phase_tx[], COMP freq[],
+void fdm_upconvert(COMP tx_fdm[], int Nc, COMP tx_baseband[NC+1][M_FAC], COMP phase_tx[], COMP freq[],
                    COMP *fbb_phase, COMP fbb_rect)
 {
     int   i,c;
     COMP  two = {2.0, 0.0};
     float mag;
 
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_FAC; i++) {
 	tx_fdm[i].real = 0.0;
 	tx_fdm[i].imag = 0.0;
     }
 
     for (c=0; c<=Nc; c++)
-	for (i=0; i<M; i++) {
+	for (i=0; i<M_FAC; i++) {
 	    phase_tx[c] = cmult(phase_tx[c], freq[c]);
 	    tx_fdm[i] = cadd(tx_fdm[i], cmult(tx_baseband[c][i], phase_tx[c]));
 	}
 
     /* shift whole thing up to carrier freq */
 
-    for (i=0; i<M; i++) {
+    for (i=0; i<M_FAC; i++) {
 	*fbb_phase = cmult(*fbb_phase, fbb_rect);
 	tx_fdm[i] = cmult(tx_fdm[i], *fbb_phase);
     }
@@ -541,7 +558,7 @@ void fdm_upconvert(COMP tx_fdm[], int Nc, COMP tx_baseband[NC+1][M], COMP phase_
       shifting for the purpose of testing easier
     */
 
-    for (i=0; i<M; i++)
+    for (i=0; i<M_FAC; i++)
 	tx_fdm[i] = cmult(two, tx_fdm[i]);
 
     /* normalise digital oscilators as the magnitude can drift over time */
@@ -597,7 +614,7 @@ void fdmdv_mod(struct FDMDV *fdmdv, COMP tx_fdm[], int tx_bits[], int *sync_bit)
   AUTHOR......: David Rowe
   DATE CREATED: 19/4/2012
 
-  Generate M samples of DBPSK pilot signal for Freq offset estimation.
+  Generate M_FAC samples of DBPSK pilot signal for Freq offset estimation.
 
 \*---------------------------------------------------------------------------*/
 
@@ -605,7 +622,7 @@ void generate_pilot_fdm(COMP *pilot_fdm, int *bit, float *symbol,
 			float *filter_mem, COMP *phase, COMP *freq)
 {
     int   i,j,k;
-    float tx_baseband[M];
+    float tx_baseband[M_FAC];
 
     /* +1 -1 +1 -1 DBPSK sync carrier, once filtered becomes (roughly)
        two spectral lines at +/- RS/2 */
@@ -618,26 +635,26 @@ void generate_pilot_fdm(COMP *pilot_fdm, int *bit, float *symbol,
     else
 	*bit = 1;
 
-    /* filter DPSK symbol to create M baseband samples */
+    /* filter DPSK symbol to create M_FAC baseband samples */
 
     filter_mem[NFILTER-1] = (sqrtf(2)/2) * *symbol;
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_FAC; i++) {
 	tx_baseband[i] = 0.0;
-	for(j=M-1,k=M-i-1; j<NFILTER; j+=M,k+=M)
-	    tx_baseband[i] += M * filter_mem[j] * gt_alpha5_root[k];
+	for(j=M_FAC-1,k=M_FAC-i-1; j<NFILTER; j+=M_FAC,k+=M_FAC)
+	    tx_baseband[i] += M_FAC * filter_mem[j] * gt_alpha5_root[k];
     }
 
     /* shift memory, inserting zeros at end */
 
-    for(i=0; i<NFILTER-M; i++)
-	filter_mem[i] = filter_mem[i+M];
+    for(i=0; i<NFILTER-M_FAC; i++)
+	filter_mem[i] = filter_mem[i+M_FAC];
 
-    for(i=NFILTER-M; i<NFILTER; i++)
+    for(i=NFILTER-M_FAC; i<NFILTER; i++)
 	filter_mem[i] = 0.0;
 
     /* upconvert */
 
-    for(i=0; i<M; i++) {
+    for(i=0; i<M_FAC; i++) {
 	*phase = cmult(*phase, *freq);
 	pilot_fdm[i].real = sqrtf(2)*2*tx_baseband[i] * phase->real;
 	pilot_fdm[i].imag = sqrtf(2)*2*tx_baseband[i] * phase->imag;
@@ -662,7 +679,7 @@ void generate_pilot_lut(COMP pilot_lut[], COMP *pilot_freq)
     float pilot_symbol = sqrtf(2.0);
     COMP  pilot_phase  = {1.0, 0.0};
     float pilot_filter_mem[NFILTER];
-    COMP  pilot[M];
+    COMP  pilot[M_FAC];
     int   i,f;
 
     for(i=0; i<NFILTER; i++)
@@ -674,7 +691,13 @@ void generate_pilot_lut(COMP pilot_lut[], COMP *pilot_freq)
     for(f=0; f<8; f++) {
 	generate_pilot_fdm(pilot, &pilot_rx_bit, &pilot_symbol, pilot_filter_mem, &pilot_phase, pilot_freq);
 	if (f >= 4)
-	    memcpy(&pilot_lut[M*(f-4)], pilot, M*sizeof(COMP));
+	    memcpy(&pilot_lut[M_FAC*(f-4)], pilot, M_FAC*sizeof(COMP));
+    }
+   
+    // create complex conjugate since we need this and only this later on 
+    for (f=0;f<4*M_FAC;f++)
+    {
+        pilot_lut[f] = cconj(pilot_lut[f]);
     }
 
 }
@@ -690,12 +713,11 @@ void generate_pilot_lut(COMP pilot_lut[], COMP *pilot_freq)
 \*---------------------------------------------------------------------------*/
 
 void lpf_peak_pick(float *foff, float *max, COMP pilot_baseband[],
-		   COMP pilot_lpf[], kiss_fft_cfg fft_pilot_cfg, COMP S[], int nin,
+		   COMP pilot_lpf[], codec2_fft_cfg fft_pilot_cfg, COMP S[], int nin,
                    int do_fft)
 {
     int   i,j,k;
     int   mpilot;
-    COMP  s[MPILOTFFT];
     float mag, imax;
     int   ix;
     float r;
@@ -703,11 +725,35 @@ void lpf_peak_pick(float *foff, float *max, COMP pilot_baseband[],
     /* LPF cutoff 200Hz, so we can handle max +/- 200 Hz freq offset */
 
     for(i=0; i<NPILOTLPF-nin; i++)
-	pilot_lpf[i] = pilot_lpf[nin+i];
+        pilot_lpf[i] = pilot_lpf[nin+i];
     for(i=NPILOTLPF-nin, j=NPILOTBASEBAND-nin; i<NPILOTLPF; i++,j++) {
-	pilot_lpf[i].real = 0.0; pilot_lpf[i].imag = 0.0;
-	for(k=0; k<NPILOTCOEFF; k++)
-	    pilot_lpf[i] = cadd(pilot_lpf[i], fcmult(pilot_coeff[k], pilot_baseband[j-NPILOTCOEFF+1+k]));
+        pilot_lpf[i].real = 0.0; pilot_lpf[i].imag = 0.0;
+
+        // STM32F4 hand optimized, this alone makes it go done from 1.6 to 1.17ms
+        // switching pilot_coeff to RAM (by removing const in pilot_coeff.h) would save
+        // another 0.11 ms at the expense of NPILOTCOEFF * 4 bytes == 120 bytes RAM
+
+        if (NPILOTCOEFF%5 == 0)
+        {
+            for(k=0; k<NPILOTCOEFF; k+=5)
+            {
+                COMP i0 = fcmult(pilot_coeff[k], pilot_baseband[j-NPILOTCOEFF+1+k]);
+                COMP i1 = fcmult(pilot_coeff[k+1], pilot_baseband[j-NPILOTCOEFF+1+k+1]);
+                COMP i2 = fcmult(pilot_coeff[k+2], pilot_baseband[j-NPILOTCOEFF+1+k+2]);
+                COMP i3 = fcmult(pilot_coeff[k+3], pilot_baseband[j-NPILOTCOEFF+1+k+3]);
+                COMP i4 = fcmult(pilot_coeff[k+4], pilot_baseband[j-NPILOTCOEFF+1+k+4]);
+
+                pilot_lpf[i] = cadd(cadd(cadd(cadd(cadd(pilot_lpf[i], i0),i1),i2),i3),i4);
+            }
+        }
+        else
+        {
+            for(k=0; k<NPILOTCOEFF; k++)
+            {
+                pilot_lpf[i] = cadd(pilot_lpf[i], fcmult(pilot_coeff[k], pilot_baseband[j-NPILOTCOEFF+1+k]));
+            }
+
+        }
     }
 
     /* We only need to do FFTs if we are out of sync.  Making them optional saves CPU in sync, which is when
@@ -723,16 +769,12 @@ void lpf_peak_pick(float *foff, float *max, COMP pilot_baseband[],
     if (do_fft) {
 
         /* decimate to improve DFT resolution, window and DFT */
-
         mpilot = FS/(2*200);  /* calc decimation rate given new sample rate is twice LPF freq */
-        for(i=0; i<MPILOTFFT; i++) {
-            s[i].real = 0.0; s[i].imag = 0.0;
-        }
         for(i=0,j=0; i<NPILOTLPF; i+=mpilot,j++) {
-            s[j] = fcmult(hanning[i], pilot_lpf[i]);
+            S[j] = fcmult(hanning[i], pilot_lpf[i]);
         }
 
-        kiss_fft(fft_pilot_cfg, (kiss_fft_cpx *)s, (kiss_fft_cpx *)S);
+        codec2_fft_inplace(fft_pilot_cfg, S);
 
         /* peak pick and convert to Hz */
 
@@ -771,30 +813,33 @@ void lpf_peak_pick(float *foff, float *max, COMP pilot_baseband[],
 
 float rx_est_freq_offset(struct FDMDV *f, COMP rx_fdm[], int nin, int do_fft)
 {
-    int  i,j;
-    COMP pilot[M+M/P];
-    COMP prev_pilot[M+M/P];
+    int  i;
+#ifndef ARM_MATH_CM4
+    int j;
+#endif
+    COMP pilot[M_FAC+M_FAC/P];
+    COMP prev_pilot[M_FAC+M_FAC/P];
     float foff, foff1, foff2;
     float   max1, max2;
 
-    assert(nin <= M+M/P);
+    assert(nin <= M_FAC+M_FAC/P);
 
     /* get pilot samples used for correlation/down conversion of rx signal */
 
     for (i=0; i<nin; i++) {
 	pilot[i] = f->pilot_lut[f->pilot_lut_index];
 	f->pilot_lut_index++;
-	if (f->pilot_lut_index >= 4*M)
+	if (f->pilot_lut_index >= 4*M_FAC)
 	    f->pilot_lut_index = 0;
 
 	prev_pilot[i] = f->pilot_lut[f->prev_pilot_lut_index];
 	f->prev_pilot_lut_index++;
-	if (f->prev_pilot_lut_index >= 4*M)
+	if (f->prev_pilot_lut_index >= 4*M_FAC)
 	    f->prev_pilot_lut_index = 0;
     }
 
     /*
-      Down convert latest M samples of pilot by multiplying by ideal
+      Down convert latest M_FAC samples of pilot by multiplying by ideal
       BPSK pilot signal we have generated locally.  The peak of the
       resulting signal is sensitive to the time shift between the
       received and local version of the pilot, so we do it twice at
@@ -806,10 +851,18 @@ float rx_est_freq_offset(struct FDMDV *f, COMP rx_fdm[], int nin, int do_fft)
 	f->pilot_baseband2[i] = f->pilot_baseband2[i+nin];
     }
 
+#ifndef ARM_MATH_CM4
     for(i=0,j=NPILOTBASEBAND-nin; i<nin; i++,j++) {
-       	f->pilot_baseband1[j] = cmult(rx_fdm[i], cconj(pilot[i]));
-	f->pilot_baseband2[j] = cmult(rx_fdm[i], cconj(prev_pilot[i]));
+       	f->pilot_baseband1[j] = cmult(rx_fdm[i], pilot[i]);
+	f->pilot_baseband2[j] = cmult(rx_fdm[i], prev_pilot[i]);
     }
+#else
+    // TODO: Maybe a handwritten mult taking advantage of rx_fdm[0] being 
+    // used twice would be faster but this is for sure faster than 
+    // the implementation above in any case.
+    arm_cmplx_mult_cmplx_f32(&rx_fdm[0].real,&pilot[0].real,&f->pilot_baseband1[NPILOTBASEBAND-nin].real,nin);
+    arm_cmplx_mult_cmplx_f32(&rx_fdm[0].real,&prev_pilot[0].real,&f->pilot_baseband2[NPILOTBASEBAND-nin].real,nin);
+#endif
 
     lpf_peak_pick(&foff1, &max1, f->pilot_baseband1, f->pilot_lpf1, f->fft_pilot_cfg, f->S1, nin, do_fft);
     lpf_peak_pick(&foff2, &max2, f->pilot_baseband2, f->pilot_lpf2, f->fft_pilot_cfg, f->S2, nin, do_fft);
@@ -840,8 +893,8 @@ void fdmdv_freq_shift(COMP rx_fdm_fcorr[], COMP rx_fdm[], float foff,
     float mag;
     int   i;
 
-    foff_rect.real = cosf(2.0*PI*foff/FS);
-    foff_rect.imag = sinf(2.0*PI*foff/FS);
+    foff_rect.real = COSF(2.0*PI*foff/FS);
+    foff_rect.imag = SINF(2.0*PI*foff/FS);
     for(i=0; i<nin; i++) {
 	*foff_phase_rect = cmult(*foff_phase_rect, foff_rect);
 	rx_fdm_fcorr[i] = cmult(rx_fdm[i], *foff_phase_rect);
@@ -864,14 +917,14 @@ void fdmdv_freq_shift(COMP rx_fdm_fcorr[], COMP rx_fdm[], float foff,
 
 \*---------------------------------------------------------------------------*/
 
-void fdm_downconvert(COMP rx_baseband[NC+1][M+M/P], int Nc, COMP rx_fdm[], COMP phase_rx[], COMP freq[], int nin)
+void fdm_downconvert(COMP rx_baseband[NC+1][M_FAC+M_FAC/P], int Nc, COMP rx_fdm[], COMP phase_rx[], COMP freq[], int nin)
 {
     int   i,c;
     float mag;
 
     /* maximum number of input samples to demod */
 
-    assert(nin <= (M+M/P));
+    assert(nin <= (M_FAC+M_FAC/P));
 
     /* downconvert */
 
@@ -897,7 +950,7 @@ void fdm_downconvert(COMP rx_baseband[NC+1][M+M/P], int Nc, COMP rx_fdm[], COMP
   DATE CREATED: 22/4/2012
 
   Receive filter each baseband signal at oversample rate P.  Filtering at
-  rate P lowers CPU compared to rate M.
+  rate P lowers CPU compared to rate M_FAC.
 
   Depending on the number of input samples to the demod nin, we
   produce P-1, P (usually), or P+1 filtered samples at rate P.  nin is
@@ -906,13 +959,13 @@ void fdm_downconvert(COMP rx_baseband[NC+1][M+M/P], int Nc, COMP rx_fdm[], COMP
 
 \*---------------------------------------------------------------------------*/
 
-void rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_baseband[NC+1][M+M/P], COMP rx_filter_memory[NC+1][NFILTER], int nin)
+void rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_baseband[NC+1][M_FAC+M_FAC/P], COMP rx_filter_memory[NC+1][NFILTER], int nin)
 {
     int c, i,j,k,l;
-    int n=M/P;
+    int n=M_FAC/P;
 
     /* rx filter each symbol, generate P filtered output samples for
-       each symbol.  Note we keep filter memory at rate M, it's just
+       each symbol.  Note we keep filter memory at rate M_FAC, it's just
        the filter output at rate P */
 
     for(i=0, j=0; i<nin; i+=n,j++) {
@@ -955,9 +1008,9 @@ void rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_baseband[NC+1][M+M/P], C
 void rxdec_filter(COMP rx_fdm_filter[], COMP rx_fdm[], COMP rxdec_lpf_mem[], int nin) {
     int i,j,k;
 
-    for(i=0; i<NRXDEC-1+M-nin; i++)
+    for(i=0; i<NRXDEC-1+M_FAC-nin; i++)
         rxdec_lpf_mem[i] = rxdec_lpf_mem[i+nin];
-    for(i=0, j=NRXDEC-1+M-nin; i<nin; i++,j++)
+    for(i=0, j=NRXDEC-1+M_FAC-nin; i<nin; i++,j++)
         rxdec_lpf_mem[j] = rx_fdm[i];
 
     for(i=0; i<nin; i++) {
@@ -970,29 +1023,100 @@ void rxdec_filter(COMP rx_fdm_filter[], COMP rx_fdm[], COMP rxdec_lpf_mem[], int
     }
 }
 
-
 /*---------------------------------------------------------------------------*\
 
-  FUNCTION....: fir_filter()
-  AUTHOR......: David Rowe
-  DATE CREATED: July 2014
+  FUNCTION....: fir_filter2()
+  AUTHOR......: Danilo Beuche
+  DATE CREATED: August 2016
 
-  Helper fir filter function.
+  Ths version submitted by Danilo for the STM32F4 platform.  The idea
+  is to avoid reading the same value from the STM32F4 "slow" flash
+  twice. 2-4ms of savings per frame were measured by Danilo and the mcHF
+  team.
 
 \*---------------------------------------------------------------------------*/
 
-static float fir_filter(float mem[], float coeff[], int dec_rate) {
-    float acc = 0.0;
-    int   m;
+static void fir_filter2(float acc[2], float mem[], const float coeff[], const unsigned int dec_rate) {
+    acc[0] = 0.0;
+    acc[1] = 0.0;
 
-    for(m=0; m<NFILTER; m+=dec_rate) {
-        acc += coeff[m] * mem[2*m];
+    float c1,c2,c3,c4,c5,m1,m2,m3,m4,m5,m6,m7,m8,m9,m10,a1,a2;
+    float* inpCmplx = &mem[0];
+    const float* coeffPtr = &coeff[0];
+
+    int m;
+
+    // this manual loop unrolling gives significant boost on STM32 machines
+    // reduction from avg 3.2ms to 2.4ms in tfdmv.c test
+    // 5 was the sweet spot, with 6 it took longer again
+    // and should not harm other, more powerful machines
+    // no significant difference in output, only rounding (which was to be expected)
+    // TODO: try to move coeffs to RAM and check if it makes a significant difference
+    if (NFILTER%(dec_rate*5) == 0) {
+        for(m=0; m<NFILTER; m+=dec_rate*5) {
+            c1 = *coeffPtr;
+
+            m1 = inpCmplx[0];
+            m2 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            c2 = *coeffPtr;
+            m3 = inpCmplx[0];
+            m4 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            c3 = *coeffPtr;
+            m5 = inpCmplx[0];
+            m6 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            c4 = *coeffPtr;
+            m7 = inpCmplx[0];
+            m8 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            c5 = *coeffPtr;
+            m9 = inpCmplx[0];
+            m10 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            a1 = c1 * m1 + c2 * m3 + c3 * m5 + c4 * m7 + c5 * m9;
+            a2 = c1 * m2 + c2 * m4 + c3 * m6 + c4 * m8 + c5 * m10;
+            acc[0] += a1;
+            acc[1] += a2;
+        }
     }
+    else
+    {
+        for(m=0; m<NFILTER; m+=dec_rate) {
+            c1 = *coeffPtr;
 
-    return dec_rate*acc;
+            m1 = inpCmplx[0];
+            m2 = inpCmplx[1];
+
+            inpCmplx+= dec_rate*2;
+            coeffPtr+= dec_rate;
+
+            a1 = c1 * m1;
+            a2 = c1 * m2;
+            acc[0] += a1;
+            acc[1] += a2;
+        }
+    }
+    acc[0] *= dec_rate;
+    acc[1] *= dec_rate;
 }
 
-
 /*---------------------------------------------------------------------------*\
 
   FUNCTION....: down_convert_and_rx_filter()
@@ -1017,21 +1141,26 @@ void down_convert_and_rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_fdm[],
                                 COMP rx_fdm_mem[], COMP phase_rx[], COMP freq[],
                                 float freq_pol[], int nin, int dec_rate)
 {
-    int i,k,c,st,N;
+    int i,k,c,st,Nval;
     float windback_phase, mag;
     COMP  windback_phase_rect;
-    COMP  rx_baseband[NFILTER+M];
+    COMP  rx_baseband[NFILTER+M_FAC];
     COMP  f_rect;
 
     //PROFILE_VAR(windback_start,  downconvert_start, filter_start);
 
     /* update memory of rx_fdm */
 
-    for(i=0; i<NFILTER+M-nin; i++)
+#if 0
+    for(i=0; i<NFILTER+M_FAC-nin; i++)
         rx_fdm_mem[i] = rx_fdm_mem[i+nin];
-    for(i=NFILTER+M-nin, k=0; i<NFILTER+M; i++, k++)
+    for(i=NFILTER+M_FAC-nin, k=0; i<NFILTER+M_FAC; i++, k++)
         rx_fdm_mem[i] = rx_fdm[k];
-
+#else
+    // this gives only 40uS gain on STM32 but now that we have, we keep it
+    memmove(&rx_fdm_mem[0],&rx_fdm_mem[nin],(NFILTER+M_FAC-nin)*sizeof(COMP));
+    memcpy(&rx_fdm_mem[NFILTER+M_FAC-nin],&rx_fdm[0],nin*sizeof(COMP));
+#endif
     for(c=0; c<Nc+1; c++) {
 
         /*
@@ -1045,18 +1174,18 @@ void down_convert_and_rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_fdm[],
 
           This means winding phase(c) back from this point to ensure
           phase continuity.
-        */
+         */
 
         //PROFILE_SAMPLE(windback_start);
         windback_phase           = -freq_pol[c]*NFILTER;
-        windback_phase_rect.real = cosf(windback_phase);
-        windback_phase_rect.imag = sinf(windback_phase);
+        windback_phase_rect.real = COSF(windback_phase);
+        windback_phase_rect.imag = SINF(windback_phase);
         phase_rx[c]              = cmult(phase_rx[c],windback_phase_rect);
         //PROFILE_SAMPLE_AND_LOG(downconvert_start, windback_start, "        windback");
 
         /* down convert all samples in buffer */
 
-        st  = NFILTER+M-1;    /* end of buffer                  */
+        st  = NFILTER+M_FAC-1;    /* end of buffer                  */
         st -= nin-1;          /* first new sample               */
         st -= NFILTER;        /* first sample used in filtering */
 
@@ -1066,7 +1195,7 @@ void down_convert_and_rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_fdm[],
         for(i=0; i<dec_rate-1; i++)
             f_rect = cmult(f_rect,freq[c]);
 
-        for(i=st; i<NFILTER+M; i+=dec_rate) {
+        for(i=st; i<NFILTER+M_FAC; i+=dec_rate) {
             phase_rx[c]    = cmult(phase_rx[c], f_rect);
             rx_baseband[i] = cmult(rx_fdm_mem[i],cconj(phase_rx[c]));
         }
@@ -1074,17 +1203,18 @@ void down_convert_and_rx_filter(COMP rx_filt[NC+1][P+1], int Nc, COMP rx_fdm[],
 
         /* now we can filter this carrier's P symbols */
 
-        N=M/P;
-        for(i=0, k=0; i<nin; i+=N, k++) {
-           #ifdef ORIG
-           rx_filt[c][k].real = 0.0; rx_filt[c][k].imag = 0.0;
+        Nval=M_FAC/P;
+        for(i=0, k=0; i<nin; i+=Nval, k++) {
+#ifdef ORIG
+            rx_filt[c][k].real = 0.0; rx_filt[c][k].imag = 0.0;
 
-           for(m=0; m<NFILTER; m++)
-               rx_filt[c][k] = cadd(rx_filt[c][k], fcmult(gt_alpha5_root[m], rx_baseband[st+i+m]));
-           #else
-           rx_filt[c][k].real = fir_filter(&rx_baseband[st+i].real, (float*)gt_alpha5_root, dec_rate);
-           rx_filt[c][k].imag = fir_filter(&rx_baseband[st+i].imag, (float*)gt_alpha5_root, dec_rate);
-           #endif
+            for(m=0; m<NFILTER; m++)
+                rx_filt[c][k] = cadd(rx_filt[c][k], fcmult(gt_alpha5_root[m], rx_baseband[st+i+m]));
+#else
+            // rx_filt[c][k].real = fir_filter(&rx_baseband[st+i].real, (float*)gt_alpha5_root, dec_rate);
+            // rx_filt[c][k].imag = fir_filter(&rx_baseband[st+i].imag, (float*)gt_alpha5_root, dec_rate);
+            fir_filter2(&rx_filt[c][k].real,&rx_baseband[st+i].real, gt_alpha5_root, dec_rate);
+#endif
         }
         //PROFILE_SAMPLE_AND_LOG2(filter_start, "        filter");
 
@@ -1155,8 +1285,8 @@ float rx_est_timing(COMP rx_symbols[],
        out single DFT at frequency 2*pi/P */
 
     x.real = 0.0; x.imag = 0.0;
-    freq.real = cosf(2*PI/P);
-    freq.imag = sinf(2*PI/P);
+    freq.real = COSF(2*PI/P);
+    freq.imag = SINF(2*PI/P);
     phase.real = 1.0;
     phase.imag = 0.0;
 
@@ -1213,13 +1343,10 @@ float qpsk_to_bits(int rx_bits[], int *sync_bit, int Nc, COMP phase_difference[]
                    COMP rx_symbols[], int old_qpsk_mapping)
 {
     int   c;
-    COMP  pi_on_4;
     COMP  d;
     int   msb=0, lsb=0;
     float ferr, norm;
 
-    pi_on_4.real = cosf(PI/4.0);
-    pi_on_4.imag = sinf(PI/4.0);
 
     /* Extra 45 degree clockwise lets us use real and imag axis as
        decision boundaries. "norm" makes sure the phase subtraction
@@ -1296,11 +1423,8 @@ void snr_update(float sig_est[], float noise_est[], int Nc, COMP phase_differenc
     float s[NC+1];
     COMP  refl_symbols[NC+1];
     float n[NC+1];
-    COMP  pi_on_4;
     int   c;
 
-    pi_on_4.real = cosf(PI/4.0);
-    pi_on_4.imag = sinf(PI/4.0);
 
     /* mag of each symbol is distance from origin, this gives us a
        vector of mags, one for each carrier. */
@@ -1485,10 +1609,10 @@ int freq_state(int *reliable_sync_bit, int sync_bit, int *state, int *timer, int
   The input signal is complex to support single sided frequency shifting
   before the demod input (e.g. fdmdv2 click to tune feature).
 
-  The number of input samples nin will normally be M ==
+  The number of input samples nin will normally be M_FAC ==
   FDMDV_SAMPLES_PER_FRAME.  However to adjust for differences in
-  transmit and receive sample clocks nin will occasionally be M-M/P,
-  or M+M/P.
+  transmit and receive sample clocks nin will occasionally be M_FAC-M_FAC/P,
+  or M_FAC+M_FAC/P.
 
 \*---------------------------------------------------------------------------*/
 
@@ -1496,9 +1620,9 @@ void fdmdv_demod(struct FDMDV *fdmdv, int rx_bits[],
 		 int *reliable_sync_bit, COMP rx_fdm[], int *nin)
 {
     float         foff_coarse, foff_fine;
-    COMP          rx_fdm_fcorr[M+M/P];
-    COMP          rx_fdm_filter[M+M/P];
-    COMP          rx_fdm_bb[M+M/P];
+    COMP          rx_fdm_fcorr[M_FAC+M_FAC/P];
+    COMP          rx_fdm_filter[M_FAC+M_FAC/P];
+    COMP          rx_fdm_bb[M_FAC+M_FAC/P];
     COMP          rx_filt[NC+1][P+1];
     COMP          rx_symbols[NC+1];
     float         env[NT*P];
@@ -1525,20 +1649,20 @@ void fdmdv_demod(struct FDMDV *fdmdv, int rx_bits[],
 
     rxdec_filter(rx_fdm_filter, rx_fdm_fcorr, fdmdv->rxdec_lpf_mem, *nin);
     down_convert_and_rx_filter(rx_filt, fdmdv->Nc, rx_fdm_filter, fdmdv->rx_fdm_mem, fdmdv->phase_rx, fdmdv->freq,
-                               fdmdv->freq_pol, *nin, M/Q);
+                               fdmdv->freq_pol, *nin, M_FAC/Q);
     PROFILE_SAMPLE_AND_LOG(rx_est_timing_start, down_convert_and_rx_filter_start, "    down_convert_and_rx_filter");
-    fdmdv->rx_timing = rx_est_timing(rx_symbols, fdmdv->Nc, rx_filt, fdmdv->rx_filter_mem_timing, env, *nin, M);
+    fdmdv->rx_timing = rx_est_timing(rx_symbols, fdmdv->Nc, rx_filt, fdmdv->rx_filter_mem_timing, env, *nin, M_FAC);
     PROFILE_SAMPLE_AND_LOG(qpsk_to_bits_start, rx_est_timing_start, "    rx_est_timing");
 
     /* Adjust number of input samples to keep timing within bounds */
 
-    *nin = M;
+    *nin = M_FAC;
 
-    if (fdmdv->rx_timing > 2*M/P)
-	*nin += M/P;
+    if (fdmdv->rx_timing > 2*M_FAC/P)
+	*nin += M_FAC/P;
 
     if (fdmdv->rx_timing < 0)
-	*nin -= M/P;
+	*nin -= M_FAC/P;
 
     foff_fine = qpsk_to_bits(rx_bits, &sync_bit, fdmdv->Nc, fdmdv->phase_difference, fdmdv->prev_rx_symbols, rx_symbols,
                              fdmdv->old_qpsk_mapping);
