@@ -98,9 +98,8 @@ float ftomel(float fHz) {
     return mel;
 }
 
-void mel_sample_freqs_kHz(float rate_K_sample_freqs_kHz[], int K)
+void mel_sample_freqs_kHz(float rate_K_sample_freqs_kHz[], int K, float mel_start, float mel_end)
 {
-    float mel_start = ftomel(200.0); float mel_end = ftomel(3700.0); 
     float step = (mel_end-mel_start)/(K-1);
     float mel;
     int k;
@@ -123,7 +122,7 @@ void mel_sample_freqs_kHz(float rate_K_sample_freqs_kHz[], int K)
 
 \*---------------------------------------------------------------------------*/
 
-void resample_const_rate_f(MODEL *model, float rate_K_vec[], float rate_K_sample_freqs_kHz[], int K)
+void resample_const_rate_f(C2CONST *c2const, MODEL *model, float rate_K_vec[], float rate_K_sample_freqs_kHz[], int K)
 {
     int m;
     float AmdB[MAX_AMP+1], rate_L_sample_freqs_kHz[MAX_AMP+1], AmdB_peak;
@@ -136,7 +135,7 @@ void resample_const_rate_f(MODEL *model, float rate_K_vec[], float rate_K_sample
         if (AmdB[m] > AmdB_peak) {
             AmdB_peak = AmdB[m];
         }
-        rate_L_sample_freqs_kHz[m] = m*model->Wo*4.0/M_PI;
+        rate_L_sample_freqs_kHz[m] = m*model->Wo*(c2const->Fs/2000.0)/M_PI;
         //printf("m: %d AmdB: %f AmdB_peak: %f  sf: %f\n", m, AmdB[m], AmdB_peak, rate_L_sample_freqs_kHz[m]);
     }
     
@@ -327,11 +326,11 @@ void interp_Wo_v(float Wo_[], int L_[], int voicing_[], float Wo1, float Wo2, in
 
 \*---------------------------------------------------------------------------*/
 
-void resample_rate_L(MODEL *model, float rate_K_vec[], float rate_K_sample_freqs_kHz[], int K)
+void resample_rate_L(C2CONST *c2const, MODEL *model, float rate_K_vec[], float rate_K_sample_freqs_kHz[], int K)
 {
    float rate_K_vec_term[K+2], rate_K_sample_freqs_kHz_term[K+2];
    float AmdB[MAX_AMP+1], rate_L_sample_freqs_kHz[MAX_AMP+1];
-   int m;
+   int m,k;
 
    /* terminate either end of the rate K vecs with 0dB points */
 
@@ -339,7 +338,7 @@ void resample_rate_L(MODEL *model, float rate_K_vec[], float rate_K_sample_freqs
    rate_K_sample_freqs_kHz_term[0] = 0.0;
    rate_K_sample_freqs_kHz_term[K+1] = 4.0;
 
-   for(int k=0; k<K; k++) {
+   for(k=0; k<K; k++) {
        rate_K_vec_term[k+1] = rate_K_vec[k];
        rate_K_sample_freqs_kHz_term[k+1] = rate_K_sample_freqs_kHz[k];
   
@@ -347,7 +346,7 @@ void resample_rate_L(MODEL *model, float rate_K_vec[], float rate_K_sample_freqs
    }
 
    for(m=1; m<=model->L; m++) {
-       rate_L_sample_freqs_kHz[m] = m*model->Wo*4.0/M_PI;
+       rate_L_sample_freqs_kHz[m] = m*model->Wo*(c2const->Fs/2000.0)/M_PI;
    }
 
    interp_para(&AmdB[1], rate_K_sample_freqs_kHz_term, rate_K_vec_term, K+2, &rate_L_sample_freqs_kHz[1], model->L);    
@@ -369,7 +368,7 @@ void resample_rate_L(MODEL *model, float rate_K_vec[], float rate_K_sample_freqs
 
 \*---------------------------------------------------------------------------*/
 
-void determine_phase(COMP H[], MODEL *model, int Nfft, codec2_fft_cfg fwd_cfg, codec2_fft_cfg inv_cfg)
+void determine_phase(C2CONST *c2const, COMP H[], MODEL *model, int Nfft, codec2_fft_cfg fwd_cfg, codec2_fft_cfg inv_cfg)
 {
     int i,m,b;
     int Ns = Nfft/2+1;
@@ -377,12 +376,12 @@ void determine_phase(COMP H[], MODEL *model, int Nfft, codec2_fft_cfg fwd_cfg, c
     float AmdB[MAX_AMP+1], rate_L_sample_freqs_kHz[MAX_AMP+1];
 
     for(m=1; m<=model->L; m++) {
-        AmdB[m] = 20.0*log10(model->A[m]);
-        rate_L_sample_freqs_kHz[m] = (float)m*model->Wo*4.0/M_PI;
+        AmdB[m] = 20.0*log10f(model->A[m]+1);
+        rate_L_sample_freqs_kHz[m] = (float)m*model->Wo*(c2const->Fs/2000.0)/M_PI;
     }
     
     for(i=0; i<Ns; i++) {
-        sample_freqs_kHz[i] = (FS/1000.0)*(float)i/Nfft;
+        sample_freqs_kHz[i] = (c2const->Fs/1000.0)*(float)i/Nfft;
     }
 
     interp_para(Gdbfk, &rate_L_sample_freqs_kHz[1], &AmdB[1], model->L, sample_freqs_kHz, Ns);
@@ -407,7 +406,8 @@ void determine_phase(COMP H[], MODEL *model, int Nfft, codec2_fft_cfg fwd_cfg, c
 
 \*---------------------------------------------------------------------------*/
 
-void newamp1_model_to_indexes(int    indexes[], 
+void newamp1_model_to_indexes(C2CONST *c2const,
+                              int    indexes[], 
                               MODEL *model, 
                               float  rate_K_vec[], 
                               float  rate_K_sample_freqs_kHz[], 
@@ -421,7 +421,7 @@ void newamp1_model_to_indexes(int    indexes[],
 
     /* convert variable rate L to fixed rate K */
 
-    resample_const_rate_f(model, rate_K_vec, rate_K_sample_freqs_kHz, K);
+    resample_const_rate_f(c2const, model, rate_K_vec, rate_K_sample_freqs_kHz, K);
 
     /* remove mean and two stage VQ */
 
@@ -448,7 +448,7 @@ void newamp1_model_to_indexes(int    indexes[],
        an unvoiced frame */
 
     if (model->voiced) {
-        int index = encode_log_Wo(model->Wo, 6);
+        int index = encode_log_Wo(c2const, model->Wo, 6);
         if (index == 0) {
             index = 1;
         }
@@ -533,7 +533,8 @@ void newamp1_indexes_to_rate_K_vec(float  rate_K_vec_[],
 
 \*---------------------------------------------------------------------------*/
 
-void newamp1_indexes_to_model(MODEL  model_[],
+void newamp1_indexes_to_model(C2CONST *c2const,
+                              MODEL  model_[],
                               COMP   H[],
                               float *interpolated_surface_,
                               float  prev_rate_K_vec_[],
@@ -562,7 +563,7 @@ void newamp1_indexes_to_model(MODEL  model_[],
     /* decode latest Wo and voicing */
 
     if (indexes[3]) {
-        Wo_right = decode_log_Wo(indexes[3], 6);
+        Wo_right = decode_log_Wo(c2const, indexes[3], 6);
         voicing_right = 1;
     }
     else {
@@ -590,8 +591,8 @@ void newamp1_indexes_to_model(MODEL  model_[],
         model_[i].L  = aL_[i];
         model_[i].voiced = avoicing_[i];
 
-        resample_rate_L(&model_[i], &interpolated_surface_[K*i], rate_K_sample_freqs_kHz, K);
-        determine_phase(&H[(MAX_AMP+1)*i], &model_[i], NEWAMP1_PHASE_NFFT, fwd_cfg, inv_cfg);
+        resample_rate_L(c2const, &model_[i], &interpolated_surface_[K*i], rate_K_sample_freqs_kHz, K);
+        determine_phase(c2const, &H[(MAX_AMP+1)*i], &model_[i], NEWAMP1_PHASE_NFFT, fwd_cfg, inv_cfg);
     }
 
     /* update memories for next time */
