@@ -69,7 +69,7 @@ float get_float(FILE * in, const char * name, char * * cursor, char * buffer, in
 
       if ( end != s )
         *cursor = end;
-        return f;
+      return f;
     }
 
     if ( fgets(buffer, size, in) == NULL ) {
@@ -209,61 +209,12 @@ void phase_experiment_destroy(struct PEXP *pexp) {
 
 \*---------------------------------------------------------------------------*/
 
-/* Bubblesort to find highest amplitude harmonics */
-
-struct AMPINDEX {
-    float amp;
-    int   index;
-};
-
-static void bubbleSort(struct AMPINDEX numbers[], int array_size)
-{
-    int i, j;
-    struct AMPINDEX temp;
-
-  for (i = (array_size - 1); i > 0; i--)
-  {
-    for (j = 1; j <= i; j++)
-    {
-	//printf("i %d j %d %f %f \n", i, j, numbers[j-1].amp, numbers[j].amp);
-      if (numbers[j-1].amp < numbers[j].amp)
-      {
-        temp = numbers[j-1];
-        numbers[j-1] = numbers[j];
-        numbers[j] = temp;
-      }
-    }
-  }
-}
-
-
-static void print_pred_error(struct PEXP *pexp, MODEL *model, int start, int end, float mag_thresh) {
-    int   i;
-    float mag;
-
-    mag = 0.0;
-    for(i=start; i<=end; i++)
-	mag += model->A[i]*model->A[i];
-    mag = 10*log10(mag/(end-start));
-
-    if (mag > mag_thresh) {
-	for(i=start; i<=end; i++) {
-	    float pred = pexp->phi_prev[i] + N*i*(model->Wo + pexp->Wo_prev)/2.0;
-	    float err = pred - model->phi[i];
-	    err = atan2(sin(err),cos(err));
-	    printf("%f\n",err);
-	}
-	//printf("\n");
-    }
-
-}
-
 
 static void predict_phases(struct PEXP *pexp, MODEL *model, int start, int end) {
     int i;
 
     for(i=start; i<=end; i++) {
-	model->phi[i] = pexp->phi_prev[i] + N*i*model->Wo;
+	model->phi[i] = pexp->phi_prev[i] + N_SAMP*i*model->Wo;
     }
 
 }
@@ -311,7 +262,7 @@ static void predict_phases_state(struct PEXP *pexp, MODEL *model, int start, int
 	}
     else
 	for(i=start; i<=end; i++) {
-	    model->phi[i] = pexp->phi_prev[i] + N*i*best_Wo;
+	    model->phi[i] = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
 	}
     printf("state %d\n", pexp->state);
 }
@@ -330,7 +281,7 @@ static void predict_phases2(struct PEXP *pexp, MODEL *model, int start, int end)
     float pred, str, diff;
 
     for(i=start; i<=end; i++) {
-	pred = pexp->phi_prev[i] + N*i*model->Wo;
+	pred = pexp->phi_prev[i] + N_SAMP*i*model->Wo;
 	str = pexp->phi1*i;
 	diff = str - pred;
 	diff = atan2(sin(diff), cos(diff));
@@ -343,14 +294,6 @@ static void predict_phases2(struct PEXP *pexp, MODEL *model, int start, int end)
 
 }
 
-static void rand_phases(MODEL *model, int start, int end) {
-    int i;
-
-    for(i=start; i<=end; i++)
-	model->phi[i] = PI*(1.0 - 2.0*(float)rand()/RAND_MAX);
-
-}
-
 static void quant_phase(float *phase, float min, float max, int bits) {
     int   levels = 1 << bits;
     int   index;
@@ -376,6 +319,64 @@ static void quant_phases(MODEL *model, int start, int end, int bits) {
     }
 }
 
+#if 0
+/* Bubblesort to find highest amplitude harmonics */
+
+struct AMPINDEX {
+    float amp;
+    int   index;
+};
+
+static void bubbleSort(struct AMPINDEX numbers[], int array_size)
+{
+    int i, j;
+    struct AMPINDEX temp;
+
+  for (i = (array_size - 1); i > 0; i--)
+  {
+    for (j = 1; j <= i; j++)
+    {
+	//printf("i %d j %d %f %f \n", i, j, numbers[j-1].amp, numbers[j].amp);
+      if (numbers[j-1].amp < numbers[j].amp)
+      {
+        temp = numbers[j-1];
+        numbers[j-1] = numbers[j];
+        numbers[j] = temp;
+      }
+    }
+  }
+}
+
+static void print_pred_error(struct PEXP *pexp, MODEL *model, int start, int end, float mag_thresh) {
+    int   i;
+    float mag;
+
+    mag = 0.0;
+    for(i=start; i<=end; i++)
+	mag += model->A[i]*model->A[i];
+    mag = 10*log10(mag/(end-start));
+
+    if (mag > mag_thresh) {
+	for(i=start; i<=end; i++) {
+	    float pred = pexp->phi_prev[i] + N_SAMP*i*(model->Wo + pexp->Wo_prev)/2.0;
+	    float err = pred - model->phi[i];
+	    err = atan2(sin(err),cos(err));
+	    printf("%f\n",err);
+	}
+	//printf("\n");
+    }
+
+}
+
+
+static void rand_phases(MODEL *model, int start, int end) {
+    int i;
+
+    for(i=start; i<=end; i++)
+	model->phi[i] = PI*(1.0 - 2.0*(float)rand()/RAND_MAX);
+
+}
+
 static void fixed_bits_per_frame(struct PEXP *pexp, MODEL *model, int m, int budget) {
     int res, finished;
 
@@ -533,7 +534,7 @@ static void top_amp(struct PEXP *pexp, MODEL *model, int start, int end, int n_h
 	if (!top) {
 	    model->phi[i] = 0.0; /* make sure */
 	    if (pred)  {
-		//model->phi[i] = pexp->phi_prev[i] + i*N*(model->Wo + pexp->Wo_prev)/2.0;
+		//model->phi[i] = pexp->phi_prev[i] + i*N_SAMP*(model->Wo + pexp->Wo_prev)/2.0;
 		model->phi[i] = i*model->phi[1];
 	    }
 	    else
@@ -558,7 +559,7 @@ static void limit_prediction_error(struct PEXP *pexp, MODEL *model, int start, i
     float pred, pred_error, error;
 
     for(i=start; i<=end; i++) {
-	pred = pexp->phi_prev[i] + N*i*(model->Wo + pexp->Wo_prev)/2.0;
+	pred = pexp->phi_prev[i] + N_SAMP*i*(model->Wo + pexp->Wo_prev)/2.0;
 	pred_error = pred - model->phi[i];
 	pred_error -= TWO_PI*floor((pred_error+PI)/TWO_PI);
 	quant_phase(&pred_error, -limit, limit, 2);
@@ -577,7 +578,7 @@ static void quant_prediction_error(struct PEXP *pexp, MODEL *model, int start, i
     float pred, pred_error;
 
     for(i=start; i<=end; i++) {
-	pred = pexp->phi_prev[i] + N*i*(model->Wo + pexp->Wo_prev)/2.0;
+	pred = pexp->phi_prev[i] + N_SAMP*i*(model->Wo + pexp->Wo_prev)/2.0;
 	pred_error = pred - model->phi[i];
 	pred_error -= TWO_PI*floor((pred_error+PI)/TWO_PI);
 
@@ -586,7 +587,6 @@ static void quant_prediction_error(struct PEXP *pexp, MODEL *model, int start, i
     }
 }
 
-
 static void print_sparse_pred_error(struct PEXP *pexp, MODEL *model, int start, int end, float mag_thresh)
 {
     int    i, index;
@@ -604,7 +604,7 @@ static void print_sparse_pred_error(struct PEXP *pexp, MODEL *model, int start,
 	}
 
 	for(i=start; i<=end; i++) {
-	    pred = pexp->phi_prev[i] + N*i*(model->Wo + pexp->Wo_prev)/2.0;
+	    pred = pexp->phi_prev[i] + N_SAMP*i*(model->Wo + pexp->Wo_prev)/2.0;
 	    error = pred - model->phi[i];
 	    error = atan2(sin(error),cos(error));
 
@@ -621,6 +621,19 @@ static void print_sparse_pred_error(struct PEXP *pexp, MODEL *model, int start,
     }
 }
 
+static void predict_phases1(struct PEXP *pexp, MODEL *model, int start, int end) {
+    int i;
+    float best_Wo;
+
+    best_Wo = refine_Wo(pexp, model, 1, model->L);
+
+    for(i=start; i<=end; i++) {
+	model->phi[i] = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
+    }
+}
+
+
+#endif
 
 static void update_snr_calc(struct PEXP *pexp, MODEL *model, float before[])
 {
@@ -750,13 +763,14 @@ static float refine_Wo(struct PEXP     *pexp,
 
     Wo_est = (model->Wo + pexp->Wo_prev)/2.0;
     best_var = 1E32;
+    best_Wo = Wo_est;
     for(Wo=0.97*Wo_est; Wo<=1.03*Wo_est; Wo+=0.001*Wo_est) {
 
 	/* predict phase and sum differences between harmonics */
 
 	var = 0.0;
 	for(i=start; i<=end; i++) {
-	    pred = pexp->phi_prev[i] + N*i*Wo;
+	    pred = pexp->phi_prev[i] + N_SAMP*i*Wo;
 	    error = pred - model->phi[i];
 	    error = atan2(sin(error),cos(error));
 	    var += error*error;
@@ -816,7 +830,7 @@ static void sparse_vq_pred_error(struct PEXP     *pexp,
 
     //printf("\n");
     for(i=1; i<=model->L; i++) {
-	pred = pexp->phi_prev[i] + N*i*best_Wo;
+	pred = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
 	error = pred - model->phi[i];
 
 	index = MAX_AMP*i*model->Wo/PI;
@@ -839,7 +853,7 @@ static void sparse_vq_pred_error(struct PEXP     *pexp,
     /* transform quantised phases back */
 
     for(i=1; i<=model->L; i++) {
-	pred = pexp->phi_prev[i] + N*i*best_Wo;
+	pred = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
 
 	index = MAX_AMP*i*model->Wo/PI;
 	assert(index < MAX_AMP);
@@ -851,18 +865,6 @@ static void sparse_vq_pred_error(struct PEXP     *pexp,
 }
 
 
-static void predict_phases1(struct PEXP *pexp, MODEL *model, int start, int end) {
-    int i;
-    float best_Wo;
-
-    best_Wo = refine_Wo(pexp, model, 1, model->L);
-
-    for(i=start; i<=end; i++) {
-	model->phi[i] = pexp->phi_prev[i] + N*i*best_Wo;
-    }
-}
-
-
 /*
   This functions tests theory that some bands can be combined together
   due to less frequency resolution at higher frequencies.  This will
@@ -890,7 +892,7 @@ void smooth_phase(struct PEXP *pexp, MODEL *model, int mode)
     /* set up sparse array */
 
     for(m=1; m<=model->L; m++) {
-	pred = pexp->phi_prev[m] + N*m*best_Wo;
+	pred = pexp->phi_prev[m] + N_SAMP*m*best_Wo;
 	err = model->phi[m] - pred;
 	err = atan2(sin(err),cos(err));
 
@@ -958,7 +960,7 @@ void smooth_phase(struct PEXP *pexp, MODEL *model, int mode)
     for(m=1; m<=model->L; m++) {
 	index = MAX_AMP*m*model->Wo/PI;
 	assert(index < MAX_AMP);
-	pred = pexp->phi_prev[m] + N*m*best_Wo;
+	pred = pexp->phi_prev[m] + N_SAMP*m*best_Wo;
 	err = atan2(sparse_pe_out[index].imag, sparse_pe_out[index].real);
 	model->phi[m] = pred + err;
     }
@@ -992,13 +994,13 @@ void smooth_phase2(struct PEXP *pexp, MODEL *model) {
 	c = s = 0.0;
 	if (h>1) {
 	    for(i=a; i<b; i++) {
-		pred = pexp->phi_prev[i] + N*i*best_Wo;
+		pred = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
 		err = model->phi[i] - pred;
 		c += cos(err); s += sin(err);
 	    }
 	    phi1_ = atan2(s,c);
 	    for(i=a; i<b; i++) {
-		pred = pexp->phi_prev[i] + N*i*best_Wo;
+		pred = pexp->phi_prev[i] + N_SAMP*i*best_Wo;
 		printf("%d: %4.3f -> ", i, model->phi[i]);
 		model->phi[i] = pred + phi1_;
 		model->phi[i] = atan2(sin(model->phi[i]),cos(model->phi[i]));
@@ -1044,6 +1046,7 @@ void smooth_phase3(struct PEXP *pexp, MODEL *model) {
 
 	    /* find bin  */
 
+	    b = MAX_BINS;
 	    for(i=0; i<nbins; i++)
 		if ((f > bins[i]) && (f <= bins[i+1]))
 		    b = i;
@@ -1051,7 +1054,7 @@ void smooth_phase3(struct PEXP *pexp, MODEL *model) {
 
 	    /* est predicted phase from average */
 
-	    pred = pexp->phi_prev[m] + N*m*best_Wo;
+	    pred = pexp->phi_prev[m] + N_SAMP*m*best_Wo;
 	    err = model->phi[m] - pred;
 	    av[b].real += cos(err); av[b].imag += sin(err);
 	}
@@ -1066,6 +1069,7 @@ void smooth_phase3(struct PEXP *pexp, MODEL *model) {
 
 	    /* find bin */
 
+            b = MAX_BINS; 
 	    for(i=0; i<nbins; i++)
 		if ((f > bins[i]) && (f <= bins[i+1]))
 		    b = i;
@@ -1075,7 +1079,7 @@ void smooth_phase3(struct PEXP *pexp, MODEL *model) {
 
 	    printf("L %d m %d f %4.f b %d\n", model->L, m, f, b);
 
-	    pred = pexp->phi_prev[m] + N*m*best_Wo;
+	    pred = pexp->phi_prev[m] + N_SAMP*m*best_Wo;
 	    err = atan2(av[b].imag, av[b].real);
 	    printf(" %d: %4.3f -> ", m, model->phi[m]);
 	    model->phi[m] = pred + err;
@@ -1115,6 +1119,7 @@ void cb_phase1(struct PEXP *pexp, MODEL *model) {
 
 	    /* find bin  */
 
+	    b = MAX_BINS;
 	    for(i=0; i<nbins; i++)
 		if ((f > bins[i]) && (f <= bins[i+1]))
 		    b = i;
@@ -1136,13 +1141,14 @@ void cb_phase1(struct PEXP *pexp, MODEL *model) {
 
 	    /* find bin */
 
+	    b = MAX_BINS;
 	    for(i=0; i<nbins; i++)
 		if ((f > bins[i]) && (f <= bins[i+1]))
 		    b = i;
 	    assert(b < MAX_BINS);
 
 	    if (m != max_ind[b])
-		model->phi[m] = pexp->phi_prev[m] + N*m*best_Wo;
+		model->phi[m] = pexp->phi_prev[m] + N_SAMP*m*best_Wo;
 	}
     }
 }
@@ -1156,7 +1162,7 @@ void cb_phase1(struct PEXP *pexp, MODEL *model) {
 
 void cb_phase2(struct PEXP *pexp, MODEL *model) {
     int   st, m, i, a, b, step;
-    float diff,w,c,s,phi1_;
+    float diff,w,c,s;
     float A[MAX_AMP];
 
     for(m=1; m<=model->L; m++) {
@@ -1166,7 +1172,7 @@ void cb_phase2(struct PEXP *pexp, MODEL *model) {
 
     st = 2*model->L/4;
     step = 3;
-    model->phi[1] = pexp->phi_prev[1] + (pexp->Wo_prev+model->Wo)*N/2.0;
+    model->phi[1] = pexp->phi_prev[1] + (pexp->Wo_prev+model->Wo)*N_SAMP/2.0;
 
     printf("L=%d ", model->L);
     for(m=st; m<st+step*2; m+=step) {
@@ -1182,12 +1188,12 @@ void cb_phase2(struct PEXP *pexp, MODEL *model) {
 	    w = 1.0;
 	    c += w*cos(diff); s += w*sin(diff);
 	}
-	phi1_ = atan2(s,c);
+	// float phi1_ = atan2(s,c);
 	printf("replacing: ");
 	for(i=a; i<b; i++) {
 	    //model->phi[i] = i*phi1_;
 	    //model->phi[i] = i*model->phi[1];
-	    //model->phi[i] = m*(pexp->Wo_prev+model->Wo)*N/2.0;
+	    //model->phi[i] = m*(pexp->Wo_prev+model->Wo)*N_SAMP/2.0;
 	    model->A[m] = A[m];
 	    printf("%d ", i);
 	}
@@ -1223,7 +1229,7 @@ static void repeat_phases(struct PEXP *pexp, MODEL *model) {
 
     *model = pexp->prev_model;
     for(m=1; m<=model->L; m++)
-	model->phi[m] += N*m*model->Wo;
+	model->phi[m] += N_SAMP*m*model->Wo;
 
 }
 
@@ -1347,7 +1353,7 @@ void phase_experiment(struct PEXP *pexp, MODEL *model, char *arg) {
     /* update states */
 
     //best_Wo = refine_Wo(pexp, model,  model->L/2, model->L);
-    pexp->phi1 += N*model->Wo;
+    pexp->phi1 += N_SAMP*model->Wo;
 
     for(m=1; m<=model->L; m++)
 	pexp->phi_prev[m] = model->phi[m];
