@@ -32,14 +32,16 @@
 #include "comp.h"
 #include "codec2_fft.h"
 
-void make_analysis_window(codec2_fft_cfg fft_fwd_cfg, float w[], COMP W[]);
+C2CONST c2const_create(int Fs);
+
+void make_analysis_window(C2CONST *c2const, codec2_fft_cfg fft_fwd_cfg, float w[], COMP W[]);
 float hpf(float x, float states[]);
-void dft_speech(codec2_fft_cfg fft_fwd_cfg, COMP Sw[], float Sn[], float w[]);
-void two_stage_pitch_refinement(MODEL *model, COMP Sw[]);
+void dft_speech(C2CONST *c2const, codec2_fft_cfg fft_fwd_cfg, COMP Sw[], float Sn[], float w[]);
+void two_stage_pitch_refinement(C2CONST *c2const, MODEL *model, COMP Sw[]);
 void estimate_amplitudes(MODEL *model, COMP Sw[], COMP W[], int est_phase);
-float est_voicing_mbe(MODEL *model, COMP Sw[], COMP W[]);
-void make_synthesis_window(float Pn[]);
-void synthesise(codec2_fftr_cfg fftr_inv_cfg, float Sn_[], MODEL *model, float Pn[], int shift);
+float est_voicing_mbe(C2CONST *c2const, MODEL *model, COMP Sw[], COMP W[]);
+void make_synthesis_window(C2CONST *c2const, float Pn[]);
+void synthesise(int n_samp, codec2_fftr_cfg fftr_inv_cfg, float Sn_[], MODEL *model, float Pn[], int shift);
 
 #define CODEC2_RAND_MAX 32767
 int codec2_rand(void);
